// $Id: args.c,v 1.11 2003/12/16 21:47:56 rwuest Exp $
/*
    SPGMR08 68HC08GP32 Loader
    handler for command line args
    Copyright (C) 2000  Robert Wuest (rwuest@wuest.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include <errno.h>
#include "spgmr.h"


#ifndef VERSION
#define VERSION "0.0"
#endif


/****************************************************************************/
/*                 the global data                                          */
/* Note that I group all related variables together, even if they aren't    */
/* necessarily initialized here.                                            */
/****************************************************************************/
int verbose = 0;
int progress = 0;

char *exename;

int debug = 0;
int noprompt = 0;

int pflag = 0;
FILE *pfile = 0;
char *pfilename = 0;

int rflag = 0;
FILE *rfile = 0;
char *rfilename = 0;

int Sflag = 0;
FILE *Sfile = 0;
char *Sfilename = 0;

int Cflag = 0;
FILE *Cfile = 0;
char *Cfilename = 0;

int eflag = 0;
int interactive = 1;

int Rflag = 0;
char *runaddress;

char *aspec[10];
int acnt = 0;

FILE *logfile;
char *logfilename = 0;
FILE *infile;
char *infilename = 0;
FILE *outfile;
char *outfilename = 0;
FILE *errfile;
char *errfilename = 0;

FILE *dbgfile;

int signum = 0;

char *portname = NULL;	// The default tty port
char *cpuname = 0;

int baud_rate = 9600;	// The default baud rate

void prusage();
char *basename (const char *name);

char *short_options =  "hl:Vvp::r::eR::a::P:S:Gc:iI:O:E:C:";
static struct option long_options[] =
{
	{"help",		no_argument,		0,'h'},	//
	{"log",			required_argument,	0,'l'}, //
	{"version",		no_argument,		0,'V'}, //
	{"verbose",		no_argument,		0,'v'}, //
	{"program",		optional_argument,	0,'p'}, //
	{"read",		optional_argument,	0,'r'}, //
	{"erase",		no_argument,		0,'e'}, //
	{"run",			optional_argument,	0,'R'}, //
	{"address",		optional_argument,	0,'a'}, //
	{"port",		required_argument,	0,'P'}, //
	{"security",	required_argument,	0,'S'}, //
	{"progress",	no_argument,		0,'G'}, //
	{"command",		required_argument,	0,'c'},
	{"interactive",	no_argument,		0,'i'},
	{"input",		required_argument,	0,'I'}, //
	{"output",		required_argument,	0,'O'}, //
	{"error",		required_argument,	0,'E'}, //
	{"debug",		no_argument,		0, 2 }, //
	{"noprompt",	no_argument,		0, 3 },
	{"signal",		required_argument,	0, 4 }, //
	{"cpu",			required_argument,	0,'C'},
	{"baud",		required_argument,	0,'b'},
	{0,0,0,0}

};


/****************************************************************************/
/*                 int process_args(int argc, char *argv[])                 */
/*                                                                          */
/* Handle the command line and init globals                                 */
/****************************************************************************/
int process_args(int argc, char *argv[])
{
	int result;
	int option_index;

	exename = basename( argv[0] );

	/* Default paths */
	infile = stdin;
	outfile = stdout;
	errfile = stderr;
	dbgfile = stderr;
	logfile = stderr;

	/* process command line */

	optind = 0;	/* initialize getopt */
	while ( (result = getopt_long( argc, argv,
											 short_options,
											 long_options,
											 &option_index)
			  ) != -1 ) {
		switch (result) {
		case 'h': /* HELP */
			prusage();
			exit(0);
		
		case 'l':
			logfilename = (char *)malloc( strlen(optarg)+1 );
			if ( !logfilename ) {
				perror( exename );
				exit( errno );
			}
			strcpy( logfilename, optarg );
			break;
		
		case 'V':
#ifdef BUILD
			xprint("%s version %s\n",exename,BUILD);
#else
			xprint("%s version %s\n",exename,VERSION);
#endif
			eexit (0);
			break;
		
		case 'v':			// verbose diagnostic messages
			verbose = 1;
			break;
		
		case 'p':			// program - with optional filename
			pflag = 1;
			interactive = 0;
			if ( optarg ) {
				pfilename = optarg;
			} else {
				pfilename = "__STDIN__";
				pfile = stdin;
			}
			break;

		case 'r':			// read chip - with optional filename
			rflag = 1;
			interactive = 0;
			if ( optarg ) {
				rfilename = optarg;
			} else {
				rfilename = "__STDOUT__";
					rfile = stdout;
			}
			break;

		case 'e':			// do erase
			eflag = 1;
			interactive = 0;
			break;

		case 'R':			// Run at optional address
			Rflag = 1;
			interactive = 0;
			if ( optarg ) {
				runaddress = optarg;
			}
			break;

		case 'a':			// address spec
			aspec[acnt] = optarg;	// save address specification
			acnt++;
			break;

		//"hl:Vvp::r::eR::a::P:S:G";
		case 'P':			// serial port name
			if (portname != NULL)
				free(portname);
			portname = strdup(optarg);
			break;

		case 'S':		// security codes filename
			Sflag = 1;
			Sfilename = optarg;
			break;

		case 'G':
			progress = 1;
			break;

		case 'c':		// execute command and exit FIXME
			Cflag = 1;
			Cfilename = optarg;
			break;

		case 'i':
			interactive = 2;
			break;

		case 'I':			// --input
			if ( infilename == 0 ) {
				infilename = optarg;
			}
			break;

		case 'O':			// --output
			if ( outfilename == 0 ) {
				outfilename = optarg;
			}
			break;

		case 'E':			// --error
			if ( errfilename == 0 ) {
				errfilename = optarg;
			}
			break;

		case 'C':
			cpuname = optarg;
			break;

		case 2:
			debug = 1;
			break;

		case 3:
			noprompt = 1;
			break;

		case 4:		// --signal
			signum = atoi(optarg);
			break;

		default:
			break;	/* just ignore bad switches - shouldn't get these anyway */
		}
	}

	/* open logfile if requested */
	if ( logfilename != 0 ) {
		if ( (logfile = fopen(logfilename,"w")) == NULL) {
			eprint( "can't open %s.\n", logfilename );
		}
		else {
			xprint( "Logging to %s\n", logfilename );
		}
	}
	/* Redirected I/O */
	if ( infilename != 0 ) {
		if ( (infile = fopen(infilename,"r")) == NULL) {
			eprint( "can't open input file: %s.\n", infilename );
			eexit(errno);
		}
	}
	if ( outfilename != 0 ) {
		if ( (outfile = fopen(outfilename,"w")) == NULL) {
			eprint( "can't open output file: %s.\n", outfilename );
			eexit(errno);
		}
	}
	if ( errfilename != 0 ) {
		if ( (errfile = fopen(errfilename,"w")) == NULL) {
			eprint( "can't open error file: %s.\n", errfilename );
			eexit(errno);
		}
	}
	if (portname == NULL)
		portname = "/dev/ttyS1";
	
	return 0;
}

/****************************************************************************/
/*                 void prusage(void)                                       */
/*                                                                          */
/* Print the usage blurb                                                    */
/****************************************************************************/
void prusage(void)
{
	xprint(
		"-p, --program [filename]    Program chip from file (stdin)\n"
		"-r, --read [filename]       Read chip saving to filename (stdout)\n"
		"-e, --erase                 Erase chip\n"
		"-R, --run [address]         Jump to code at address (reset)\n"
		"-P, --port devname          Use devname as serial port (/dev/ttyS0)\n"
		"-b, --baud rate             Specify baud rate\n"
		"-S, --security filename     Get/save security codes to filename\n"
		"-G, --progress              Show progress hashes during action\n"
		"-a, --address address_spec  Add an address specification.\n"
		"                            start:end or start+len\n"
		"-I, --input filename        File to read for interpreter input\n"
		"-O, --output filename       File to write output to\n"
		"-E, --error filename        File to write error messages to\n"
		"    --signal signum         Send signal signum  on writes\n"
		"-i, --interactive           Force interactive mode, even if not\n"
		"                            talking to a tty\n"
//      "-c, --comand "command"      Pass command to the interpreter, execute and exit\n"
// undocumented:		"    --debug               Pront some debugging info\n"
// undocumented:		"-v, --verbose             Be verbose\n"
// undocumented:		"-l, --log filename        write debug to logfile\n"
		"-V, --version               Print version and exit\n"
		"-h, --help                  This help message\n"
	);
}

// $Log: args.c,v $
// Revision 1.11  2003/12/16 21:47:56  rwuest
// patches from Anders Nystrom <andesr@southpole.se>
// add jb8 processor
// add patch target
// fix default port bug
// added load_module()
//
// Revision 1.10  2003/12/16 20:58:40  rwuest
// changed unsigned chars to char
// renamed baudrate to baud_rate
//
// Revision 1.9  2001/08/17 11:54:43  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.8  2001/08/11 14:29:32  rwuest
// Added -b, --baud nnnn option
// added global int baudrate
//
// Revision 1.7  2001/08/04 16:52:47  rwuest
// added --command and --cpu command line options
// removed -C, --sec-sodes (just use interactive mode or --command)
//
// Revision 1.6  2001/07/29 03:14:20  rwuest
// added --signal signum to command line, will signal parent in xprint,
// Really for debugging garbage in the pipes.
//
// Revision 1.5  2001/07/27 16:43:49  rwuest
// uniform error print statements
//
// Revision 1.4  2001/07/22 02:18:35  rwuest
// version output goes through xprint, duh!
//
// Revision 1.3  2001/07/22 02:02:54  rwuest
// fix -V option and BUILD variable
// use printf and exit.
//
// Revision 1.2  2001/07/22 00:34:28  rwuest
// Added support fo named pipes: infile, outfile, errfile
// Debug messages now all print through dprint()
// Standard out now all goes througn xprint()
// Error messages all go through eprint()
//
// Revision 1.1  2001/07/19 21:24:53  rwuest
// Initial checkin - released as 0.1
//

