/*
 * A n t l r  T r a n s l a t i o n  H e a d e r
 *
 * Terence Parr, Will Cohen, and Hank Dietz: 1989-2001
 * Purdue University Electrical Engineering
 * With AHPCRC, University of Minnesota
 * ANTLR Version 1.33MR33
 *
 *   antlr -gk -w2 -gh -gl -e2 cli.g
 *
 */

#define ANTLR_VERSION	13333
#include "pcctscfg.h"
#include "pccts_stdio.h"
#line 1 "cli.g"


typedef union {
  long long ival;
  char *hseq;		/*	hex sequence data is malloc'ed to this ptr */
  char *strdata;
  char str[128];
} Attrib;
void zzcr_attr(Attrib *attr, int type, char *text );
#define USER_ZZSYN
#define DEMAND_LOOK

#define zzSET_SIZE 8
#include "antlr.h"
#include "tokens.h"
#include "dlgdef.h"
#include "mode.h"

/* MR23 In order to remove calls to PURIFY use the antlr -nopurify option */

#ifndef PCCTS_PURIFY
#define PCCTS_PURIFY(r,s) memset((char *) &(r),'\0',(s));
#endif

ANTLR_INFO
#line 45 "cli.g"

#include <stdio.h>
#include <termios.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <setjmp.h>
#ifdef __cpluspluss
extern "C" {
#endif
#include <readline/readline.h>
#include <readline/history.h>
#ifdef __cpluspluss
}
#endif


#include "spgmr.h"
#include "s1utils.h"
#include "cli.h"
#include "mongp32.h"
#include "lin_serial.h"
#include "seccodes.h"
#include "ops.h"

#ifndef DEBUG_READLINE
#define DBGRDL(X)
#else
#define DBGRDL(X) if(debug){X}
#endif

int dbgpar = 0;
#ifndef DEBUG_PARSER
#define DBGPAR(X)
#else
#define DBGPAR(X) X
#endif

struct sigaction cli_act;

static volatile sig_atomic_t done=0;				// flag to exit cli read loop
sigjmp_buf env;			// jump buffer to handle interrupted readline
int sourcing = 0;		// disbles quit command while sourcing

#ifdef USE_SCOPE
char *scopecmd = "xterm -e scope -b9600&";
#else
char *scopecmd = 0;
#endif

// current CPU
static CPU *clicpu=0;

// function defs
static int do_shell( char *cline );
static void noerr(const char *e);
static void cli_int(int i);	// cli interrupt handler
static void sf_int(int i);	// sourcing a file interrupt handler



void cli()
{
  char *rbuffer;
  static char *promptText;		// static to avoid gcc setjmp warning.  Bug??
  
	zzerr = noerr;
  
	cli_act.sa_handler = cli_int;
  cli_act.sa_flags = SA_RESTART;
  sigemptyset(&cli_act.sa_mask);
  sigaction(SIGINT,&cli_act,NULL);	// clean ^c exit
  
//	rl_catch_signals = 1;
  
	if ( isatty(fileno(infile)) | (interactive>1) ) {
    promptText = "spgmr: ";	
    rl_readline_name = "spgmr08";
    using_history();
    read_history( SPGMRHISTORY );	// read the past history, ignore errors
    
		do {
      rbuffer = 0;
      sigsetjmp(env, 1);
      rbuffer = readline(promptText);
      DBGRDL(dprint( "readline returned: %s\n", rbuffer?rbuffer:"(NULL)");)
      if ( rbuffer) {
        if ( rbuffer[0] ) {
          add_history( rbuffer );
          if ( rbuffer[0] == '!' ) {		// handle a shell right here
            do_shell( &rbuffer[1] );
          } else {
            ANTLRs( cmd_line(), rbuffer );
          }
        }
        free (rbuffer);
      } else {			// EOF on empty line
        done = 1;
      }
    } while ( !done );
    
		write_history( SPGMRHISTORY );
    xprint("Bye!\n" );
  } else {						// non-interactive server mode
    int ll;
    
		errfile = stdout;
    promptText = "spgmr:";	
    rbuffer = (char *)malloc(2048);
    interactive = 0;
    
		do {
      //int n;
      xprint( "%s\n", promptText );
#if 1
      if ( fgets(rbuffer, 2048, infile ) ){
#else
        if ( (n = read( STDIN_FILENO, rbuffer, 2048)) ) {
          dprint( "read: %d\n", n );
          if ( n == -1 ) {
            perror("read");
          }
          rbuffer[n] = '\0';
#endif
          ll = strlen( rbuffer );
          if ( rbuffer[ll-1] == '\n' )
          rbuffer[ll-1] = '\0';
          if (debug) {
            dprint( "fgets: %s\n", rbuffer);
          }
          if ( rbuffer[0] ) {
            if ( rbuffer[0] == '!' ) {		// handle a shell right here
              do_shell( &rbuffer[1] );
            } else {
              ANTLRs( cmd_line(), rbuffer );
            }
          }
        } else {			// EOF on empty line
          done = 1;
        }
      } while ( !done );
    }
    
}
  

  int parse_line(char *cmd)		// externa/ interface to process a string
  {
    return ANTLRs( cmd_line(), cmd );
  }
  
int do_shell( char *cline )
  {
    char *shell;
    
	if (cline) {
      while ( *cline && (*cline == ' ') )
      cline++;
      if ( *cline ) {
        system( cline );
      } else if ( isatty(1) ) {
        shell = getenv("SHELL");
        shell = shell?shell:(char *)"/bin/sh";
        xprint( "exit or end-of-file to return to spgmr\n" );
        system(shell);
      }
      return 0;
    } else {
      return -1;
    }
  }
  
int do_scope( void )
  {
    int pid;
    char *args[] = {
      "xterm",
      "-e",
      "scope",
      "-b9600",
      0
    };
    
	if ( (pid = fork()) == 0 ) {	// the child
      execvp( "xterm" , args );
    }
    
	return pid;
    
}
  
void cli_int(int i)	// cli interrupt handler
  {
    if ( interactive ) {
      //		rl_crlf();
      rl_reset_line_state();
    }
    //	siglongjmp( env, 1 );
  }
  
void sf_int(int i)	// sourcing a file interrupt handler
  {
  }
  
void noerr(const char *e)
  {
  }
  

  void zzcr_attr(Attrib *attr, int type, char *text )
  {
    char *t;
    t = text;
    switch( type )	{
      case INT:
      attr->ival = atoi(text);
      break;
      case MINT:
      attr->ival = atoi(text);
      break;
      case MOTINT:
      t++;	/* pass $ */
      attr->ival = (int)strtol(t, NULL, 16);
      break;
      case CINT:
      t+=2;	/* pass 0x */
      attr->ival = (int)strtol(t, NULL, 16);
      break;
      //	case SNAME:
      //		t++; /* skip leading '{'  */
      //		strncpy(attr->str,t,127);
      //		attr->str[strlen(attr->str) - 1] = '\0';	/* skip trailing '}' */
      //		break;
      case STR:
      //printf("STR %s\n", text);
      /* this is definitely a memory leak */
      /* should be no big deal unless you use a lot of these */
      /* or free the space when done with it */
      /* Strip leading spaces */
      while( *t && *t==' ' )
      t++;
      /* maybe strip the quote marks */
      if ( *t =='\"' ) {
        t++;
        text[ strlen(text)-1 ] = '\0';
      }
      attr->strdata = strdup( t );
      break;
      default:
      attr->ival = atoi(text);
      break;
    }
  }
  
/* modified error reporting function */
  /*error reporting is partly broken - this needs fixin' */
#define WSYNBUFSIZE	 1024
  void zzsyn(char *text, int tok, char *egroup, SetWordType *eset, int etok, int k, char *bad_text)
  {
  char work[WSYNBUFSIZE];	// let's hope an error message is never bigger than this
  // need to do an atomic all to eprint
  int place = 0;
  int left = WSYNBUFSIZE;
  
    zzSyntaxErrCount++;                             /* MR11 */
  place = snprintf( work, WSYNBUFSIZE-place, "line %d: syntax error at \"%s\"", zzline, (tok==zzEOF_TOKEN)?"EOF":bad_text);
  if ( !etok && !eset ) {
    eprint( "%s\n", work );
    return;
  }
  if ( k==1 ) {
    place += snprintf( &work[place], WSYNBUFSIZE-place, " missing" );
  } else {
    place += snprintf( &work[place], WSYNBUFSIZE-place, "; \"%s\" not", bad_text );
    if ( zzset_deg(eset)>1 )
    place += snprintf( &work[place], WSYNBUFSIZE-place, " in" );
  }
  if ( zzset_deg(eset)>0 )
  //		place += myzzedecode(eset, &work[place]);
  zzedecode(eset);
  else
  place += snprintf( &work[place], WSYNBUFSIZE-place, " %s", zztokens[etok] );
  if ( strlen(egroup) > 0 )
  place += snprintf( &work[place], WSYNBUFSIZE-place, " in %s", egroup);
  
	eprint("%s\n", work);
}

/*   END OF C CODE   */

void
#ifdef __USE_PROTOS
cmd_file(void)
#else
cmd_file()
#endif
{
#line 428 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 429 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    while ( (setwd1[LA(1)]&0x1) ) {
#line 429 "cli.g"
      {
        zzBLOCK(zztasp3);
        zzMake0;
        {
        LOOK(1);
        if ( (setwd1[LA(1)]&0x2) ) {
#line 429 "cli.g"
          cmd_linei();
        }
        else {
          if ( (LA(1)==4) ) {
          }
          else {zzFAIL(1,zzerr1,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
        zzEXIT(zztasp3);
        }
      }
#line 429 "cli.g"
      zzmatch(4);
      zzLOOP(zztasp2);
      LOOK(1);
    }
    zzEXIT(zztasp2);
    }
  }
#line 429 "cli.g"
  zzmatch(End);
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd1, 0x4);
  }
}

void
#ifdef __USE_PROTOS
cmd_line(void)
#else
cmd_line()
#endif
{
#line 434 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 435 "cli.g"
  cmd_linei();
#line 435 "cli.g"
  zzmatch(End);
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd1, 0x8);
  }
}

void
#ifdef __USE_PROTOS
cmd_linei(void)
#else
cmd_linei()
#endif
{
#line 439 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 440 "cli.g"
  command();
#line 440 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    while ( (LA(1)==57) ) {
#line 440 "cli.g"
      zzmatch(57);
#line 440 "cli.g"
      {
        zzBLOCK(zztasp3);
        zzMake0;
        {
        LOOK(1);
        if ( (setwd1[LA(1)]&0x10) ) {
#line 440 "cli.g"
          command();
        }
        else {
          if ( (setwd1[LA(1)]&0x20) ) {
          }
          else {zzFAIL(1,zzerr2,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
        zzEXIT(zztasp3);
        }
      }
      zzLOOP(zztasp2);
      LOOK(1);
    }
    zzEXIT(zztasp2);
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd1, 0x40);
  }
}

void
#ifdef __USE_PROTOS
command(void)
#else
command()
#endif
{
#line 444 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
  LOOK(1);
  if ( (LA(1)==READ) ) {
#line 445 "cli.g"
    zzmatch(READ);
#line 445 "cli.g"
    read_cmd();
  }
  else {
    if ( (LA(1)==WRITE) ) {
#line 446 "cli.g"
      zzmatch(WRITE);
#line 446 "cli.g"
      write_cmd();
    }
    else {
      if ( (LA(1)==ERASE) ) {
#line 447 "cli.g"
        zzmatch(ERASE);
#line 447 "cli.g"
        erase_cmd();
      }
      else {
        if ( (LA(1)==PROGRAM) ) {
#line 448 "cli.g"
          zzmatch(PROGRAM);
#line 448 "cli.g"
          program_cmd();
        }
        else {
          if ( (LA(1)==VERIFY) ) {
#line 449 "cli.g"
            zzmatch(VERIFY);
#line 449 "cli.g"
            verify_cmd();
          }
          else {
            if ( (LA(1)==QUIT) ) {
#line 450 "cli.g"
              zzmatch(QUIT);
#line 450 "cli.g"
              quit_cmd();
            }
            else {
              if ( (LA(1)==RESET) ) {
#line 451 "cli.g"
                zzmatch(RESET);
#line 451 "cli.g"
                reset_cmd();
              }
              else {
                if ( (LA(1)==SCODES) ) {
#line 452 "cli.g"
                  zzmatch(SCODES);
#line 452 "cli.g"
                  scodes_cmd();
                }
                else {
                  if ( (LA(1)==WCODES) ) {
#line 453 "cli.g"
                    zzmatch(WCODES);
#line 453 "cli.g"
                    wcodes_cmd();
                  }
                  else {
                    if ( (LA(1)==TDEBUG) ) {
#line 454 "cli.g"
                      zzmatch(TDEBUG);
#line 454 "cli.g"
                      debug_cmd();
                    }
                    else {
                      if ( (LA(1)==SERIAL) ) {
#line 455 "cli.g"
                        zzmatch(SERIAL);
#line 455 "cli.g"
                        serial_cmd();
                      }
                      else {
                        if ( (LA(1)==COMMENT) ) {
#line 456 "cli.g"
                          zzmatch(COMMENT);
                        }
                        else {
                          if ( (LA(1)==SCOPE) ) {
#line 457 "cli.g"
                            zzmatch(SCOPE);
#line 457 "cli.g"
                            scope_cmd();
                          }
                          else {
                            if ( (LA(1)==SET) ) {
#line 458 "cli.g"
                              zzmatch(SET);
#line 458 "cli.g"
                              set_cmd();
                            }
                            else {
                              if ( (LA(1)==VERSN) ) {
#line 459 "cli.g"
                                zzmatch(VERSN);
#line 459 "cli.g"
                                version_cmd();
                              }
                              else {
                                if ( (LA(1)==SOURCE) ) {
#line 460 "cli.g"
                                  zzmatch(SOURCE);
#line 460 "cli.g"
                                  source_cmd();
                                }
                                else {
                                  if ( (LA(1)==RUN) ) {
#line 461 "cli.g"
                                    zzmatch(RUN);
#line 461 "cli.g"
                                    run_cmd();
                                  }
                                  else {
                                    if ( (LA(1)==TCPU) ) {
#line 462 "cli.g"
                                      zzmatch(TCPU);
#line 462 "cli.g"
                                      cpu_cmd();
                                    }
                                    else {
                                      if ( (LA(1)==SEND) ) {
#line 463 "cli.g"
                                        zzmatch(SEND);
#line 463 "cli.g"
                                        send_cmd( 0 );
                                      }
                                      else {
                                        if ( (LA(1)==SENDE1) ) {
#line 464 "cli.g"
                                          zzmatch(SENDE1);
#line 464 "cli.g"
                                          send_cmd( 1 );
                                        }
                                        else {
                                          if ( (LA(1)==SENDE2) ) {
#line 465 "cli.g"
                                            zzmatch(SENDE2);
#line 465 "cli.g"
                                            send_cmd( 2 );
                                          }
                                          else {
                                            if ( (LA(1)==DTR) ) {
#line 466 "cli.g"
                                              zzmatch(DTR);
#line 466 "cli.g"
                                              dtr_cmd();
                                            }
                                            else {zzFAIL(1,zzerr3,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
                                          }
                                        }
                                      }
                                    }
                                  }
                                }
                              }
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd1, 0x80);
  }
}

void
#ifdef __USE_PROTOS
read_cmd(void)
#else
read_cmd()
#endif
{
#line 470 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 470 "cli.g"
  int a1,a2; char *buf=0; FILE *f=0; struct HC08stack stack;
  LOOK(1);
  if ( (setwd2[LA(1)]&0x1) ) {
#line 471 "cli.g"
    {
      zzBLOCK(zztasp2);
      zzMake0;
      {
#line 471 "cli.g"
      { struct _rv30 _trv; _trv = address_expression();

      a1 = _trv.base; a2  = _trv.count; }
#line 471 "cli.g"
      {
        zzBLOCK(zztasp3);
        zzMake0;
        {
        LOOK(1);
        if ( (LA(1)==TO) ) {
#line 471 "cli.g"
          zzmatch(TO);
#line 471 "cli.g"
           f  = openfile( "w" );

        }
        else {
          if ( (setwd2[LA(1)]&0x2) ) {
          }
          else {zzFAIL(1,zzerr4,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
        zzEXIT(zztasp3);
        }
      }
      zzEXIT(zztasp2);
      }
    }
#line 471 "cli.g"
    
    DBGPAR ( if(dbgpar) dprint( "read 0x%X+0x%X\n", a1, a2 ); )
    if ( a2>0 ) {
      buf = (char *)malloc(a2);
    }
    
			if ( a2>0 )	{
      mon_blkread( sfd, buf, a1, a2);
    }
    
			if ( !f ) {
      format_dump( buf, a1, a2 );
    } else {						// TO file
      fprints0( f, "SPGMR");
      fprints1( f, buf, a1, a2 );
      fprints9( f, 0 );
      fclose( f );
    }
    if (buf) {
      free(buf);
    }
  }
  else {
    if ( (LA(1)==SP) ) {
#line 493 "cli.g"
      zzmatch(SP);
#line 493 "cli.g"
      
      buf = (char *)malloc(2);
      mon_readsp(sfd, buf);
      xprint( "SP = 0x%02X%02X\n", buf[0], buf[1] );
      free( buf );
      
    }
    else {
      if ( (LA(1)==STACK) ) {
#line 499 "cli.g"
        zzmatch(STACK);
#line 499 "cli.g"
        
        mon_show_stack( sfd, &stack);
        
      }
      else {zzFAIL(1,zzerr5,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd2, 0x4);
  }
}

void
#ifdef __USE_PROTOS
write_cmd(void)
#else
write_cmd()
#endif
{
#line 517 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 517 "cli.g"
  int a1,cnt,v; char *buf; char *fn=0; unsigned long address; int s1_len;
  LOOK(1);
  if ( (setwd2[LA(1)]&0x8) ) {
#line 518 "cli.g"
    { struct _rv30 _trv; _trv = address_expression();

    a1 = _trv.base; cnt  = _trv.count; }
#line 519 "cli.g"
    {
      zzBLOCK(zztasp2);
      zzMake0;
      {
#line 519 "cli.g"
       v  = integer();

#line 519 "cli.g"
      
      cnt = 1;
      if ( v>0xff) {
        cnt = 2;
      }
      buf = (char *)malloc(cnt);
      buf[cnt-1] = v;
      if (cnt==2)
      buf[0] = (v>>8) & 0xff;
      mon_blkwrite( sfd, buf, a1, cnt );
      free(buf);
      zzEXIT(zztasp2);
      }
    }
  }
  else {
    if ( (LA(1)==FROM) ) {
#line 532 "cli.g"
      zzmatch(FROM);
#line 532 "cli.g"
       fn  = filename();

#line 532 "cli.g"
      if(fn) { mon_write_file( fn );  free(fn); }
    }
    else {zzFAIL(1,zzerr6,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd2, 0x10);
  }
}

void
#ifdef __USE_PROTOS
erase_cmd(void)
#else
erase_cmd()
#endif
{
#line 536 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 536 "cli.g"
  int x,a,c; int bflag = 0;
#line 537 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    if ( (setwd2[LA(1)]&0x20) ) {
#line 537 "cli.g"
      { struct _rv30 _trv; _trv = address_expression();

      a = _trv.base; c  = _trv.count; }
#line 537 "cli.g"
      bflag = 1;
    }
    else {
      if ( (setwd2[LA(1)]&0x40) ) {
      }
      else {zzFAIL(1,zzerr7,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
    zzEXIT(zztasp2);
    }
  }
#line 537 "cli.g"
  
  if ( !clicpu ) {
    eprint( "Must set cpu\n");
  } else {
    if ( !bflag ) {
      mass_erase(clicpu);
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd2, 0x80);
  }
}

void
#ifdef __USE_PROTOS
program_cmd(void)
#else
program_cmd()
#endif
{
#line 549 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 549 "cli.g"
  int x,a,c; int bflag = 0; char *fn=0;
#line 550 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    if ( (setwd3[LA(1)]&0x1) ) {
#line 550 "cli.g"
      { struct _rv30 _trv; _trv = address_expression();

      a = _trv.base; c  = _trv.count; }
#line 550 "cli.g"
      bflag = 1;
    }
    else {
      if ( (LA(1)==FROM) ) {
      }
      else {zzFAIL(1,zzerr8,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
    zzEXIT(zztasp2);
    }
  }
#line 550 "cli.g"
  zzmatch(FROM);
#line 550 "cli.g"
   fn  = filename();

#line 550 "cli.g"
  
  if ( !clicpu ) {
    eprint( "Must set cpu\n");
  } else {
    if ( !bflag && fn && *fn ) {
      program_flash(clicpu,fn);
    }
  }
  if(fn)
  free(fn);
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd3, 0x2);
  }
}

void
#ifdef __USE_PROTOS
verify_cmd(void)
#else
verify_cmd()
#endif
{
#line 564 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 564 "cli.g"
  int x,a,c; int bflag = 0; char *fn=0;
#line 565 "cli.g"
  zzmatch(FROM);
#line 565 "cli.g"
   fn  = filename();

#line 565 "cli.g"
  
  if ( !clicpu ) {
    eprint( "Must set cpu\n");
  } else {
    if (fn && *fn) {
      verify_flash(clicpu,fn);
    }
  }
  if(fn)
  free(fn);
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd3, 0x4);
  }
}

void
#ifdef __USE_PROTOS
quit_cmd(void)
#else
quit_cmd()
#endif
{
#line 579 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 580 "cli.g"
  
  if ( !sourcing )
  done = 1;
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd3, 0x8);
  }
}

void
#ifdef __USE_PROTOS
reset_cmd(void)
#else
reset_cmd()
#endif
{
#line 587 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 588 "cli.g"
  
  mon_reset(sfd);
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd3, 0x10);
  }
}

void
#ifdef __USE_PROTOS
scodes_cmd(void)
#else
scodes_cmd()
#endif
{
#line 595 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
  LOOK(1);
  if ( (LA(1)==SEND) ) {
#line 596 "cli.g"
    zzmatch(SEND);
#line 596 "cli.g"
    scodes_send_();
  }
  else {
    if ( (LA(1)==SET) ) {
#line 597 "cli.g"
      zzmatch(SET);
#line 597 "cli.g"
      scodes_set_();
    }
    else {
      if ( (LA(1)==SHOW) ) {
#line 598 "cli.g"
        zzmatch(SHOW);
#line 598 "cli.g"
        scodes_show_();
      }
      else {
        if ( (LA(1)==PUSH) ) {
#line 599 "cli.g"
          zzmatch(PUSH);
#line 599 "cli.g"
          scodes_push_();
        }
        else {
          if ( (LA(1)==TRY) ) {
#line 602 "cli.g"
            zzmatch(TRY);
#line 602 "cli.g"
            scodes_try_();
          }
          else {zzFAIL(1,zzerr9,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
      }
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd3, 0x20);
  }
}

void
#ifdef __USE_PROTOS
scodes_send_(void)
#else
scodes_send_()
#endif
{
#line 609 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 609 "cli.g"
  FILE *f=0; int cnt; char buf[SECCSIZE];
  LOOK(1);
  if ( (LA(1)==FROM) ) {
#line 610 "cli.g"
    zzmatch(FROM);
#line 610 "cli.g"
     f  = openfile( "r" );

#line 610 "cli.g"
    
    if( f ) {
      cnt = scodes_find_s1( f, 0, buf );
      fclose( f );
      if (cnt == 0 ) {
        scodes_push( buf );
        mon_write_scode(sfd, 0);
        scodes_print(0);
      }
    }
  }
  else {
    if ( (LA(1)==AS) ) {
#line 621 "cli.g"
      zzmatch(AS);
#line 621 "cli.g"
       cnt  = scode_spec( buf );

#line 621 "cli.g"
      scodes_push( buf ); mon_write_scode(sfd, 0);
    }
    else {
      if ( (setwd3[LA(1)]&0x40) ) {
#line 622 "cli.g"
         cnt  = integer();

#line 622 "cli.g"
        
        if (cnt>=0 && cnt<SECCMAX )
        mon_write_scode(sfd, cnt);
      }
      else {
        if ( (LA(1)==BLANK) ) {
#line 626 "cli.g"
          zzmatch(BLANK);
#line 626 "cli.g"
          mon_write_scode(sfd, -1);
          
        }
        else {zzFAIL(1,zzerr10,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
      }
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd3, 0x80);
  }
}

void
#ifdef __USE_PROTOS
scodes_set_(void)
#else
scodes_set_()
#endif
{
#line 631 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 631 "cli.g"
  FILE *f=0; char *fn=0; int cnt; int which = 0; char buf[8];
  LOOK(1);
  if ( (LA(1)==FROM) ) {
#line 632 "cli.g"
    zzmatch(FROM);
#line 632 "cli.g"
     f  = openfile( "r" );

#line 632 "cli.g"
    
    if ( f ) {
      cnt = scodes_find_s1( f, 0, buf );
      fclose( f );
      if (cnt == 0 ) {
        scodes_push( buf );
        scodes_print(0);
      }
    }
  }
  else {
    if ( (setwd4[LA(1)]&0x1) ) {
#line 642 "cli.g"
       which  = integer();

#line 642 "cli.g"
      zzmatch(AS);
#line 642 "cli.g"
       cnt  = scode_spec( buf );

#line 642 "cli.g"
      scodes_set( buf , which);
    }
    else {zzFAIL(1,zzerr11,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd4, 0x2);
  }
}

void
#ifdef __USE_PROTOS
scodes_show_(void)
#else
scodes_show_()
#endif
{
#line 649 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 649 "cli.g"
  FILE *f=0; int cnt; char buf[SECCSIZE];
  LOOK(1);
  if ( (LA(1)==FROM) ) {
#line 650 "cli.g"
    zzmatch(FROM);
#line 650 "cli.g"
     f  = openfile( "r" );

#line 650 "cli.g"
    
    if ( f ) {
      cnt = scodes_find_s1( f, 0, buf );
      fclose( f );
      if (cnt == 0 ) {
        int ii;
        for( ii=0; ii<SECCSIZE; ii++ )
        xprint( "%02X ", buf[ii] );
        xprint("\n");
      }
    }
  }
  else {
    if ( (setwd4[LA(1)]&0x4) ) {
#line 662 "cli.g"
       cnt  = integer();

#line 662 "cli.g"
      scodes_print(cnt);
    }
    else {
      if ( (LA(1)==ALL) ) {
#line 663 "cli.g"
        zzmatch(ALL);
#line 663 "cli.g"
        
        for(cnt=0;cnt<SECCMAX;cnt++)
        scodes_print(cnt);
        
      }
      else {zzFAIL(1,zzerr12,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd4, 0x8);
  }
}

void
#ifdef __USE_PROTOS
scodes_push_(void)
#else
scodes_push_()
#endif
{
#line 672 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 672 "cli.g"
  FILE *f=0; int cnt; char buf[SECCSIZE];
  LOOK(1);
  if ( (LA(1)==FROM) ) {
#line 673 "cli.g"
    zzmatch(FROM);
#line 673 "cli.g"
     f  = openfile( "r" );

#line 673 "cli.g"
    
    if ( f ) {
      cnt = scodes_find_s1( f, 0, buf );
      fclose( f );
      if (cnt == 0 ) {
        int ii;
        for( ii=0; ii<SECCSIZE; ii++ )
        xprint( "%02X ", buf[ii] );
        xprint("\n");
      }
    }
  }
  else {
    if ( (setwd4[LA(1)]&0x10) ) {
#line 685 "cli.g"
       cnt  = integer();

#line 685 "cli.g"
      scodes_print(cnt);
    }
    else {
      if ( (LA(1)==ALL) ) {
#line 686 "cli.g"
        zzmatch(ALL);
#line 686 "cli.g"
        
        for(cnt=0;cnt<SECCMAX;cnt++)
        scodes_print(cnt);
        
      }
      else {zzFAIL(1,zzerr13,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd4, 0x20);
  }
}

void
#ifdef __USE_PROTOS
scodes_try_(void)
#else
scodes_try_()
#endif
{
#line 696 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 696 "cli.g"
  FILE *f=0; int cnt; unsigned char buf[8];
  LOOK(1);
  if ( (setwd4[LA(1)]&0x40) ) {
#line 697 "cli.g"
    {
      zzBLOCK(zztasp2);
      zzMake0;
      {
      LOOK(1);
      if ( (setwd4[LA(1)]&0x80) ) {
#line 697 "cli.g"
         cnt  = integer();

      }
      else {
        if ( (LA(1)==BLANK) ) {
#line 697 "cli.g"
          zzmatch(BLANK);
#line 697 "cli.g"
          cnt = -1;
          
        }
        else {zzFAIL(1,zzerr14,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
      }
      zzEXIT(zztasp2);
      }
    }
#line 697 "cli.g"
    
    if (!clicpu)
    eprint( "Must set cpu\n");
    else {
      cnt = reset_cpu( clicpu, cnt );
      xprint( cnt ? "failed\n" : "passed\n" );
    }
  }
  else {
    if ( (LA(1)==ALL) ) {
#line 705 "cli.g"
      zzmatch(ALL);
    }
    else {
      if ( (LA(1)==FROM) ) {
#line 706 "cli.g"
        zzmatch(FROM);
#line 706 "cli.g"
         f  = openfile( "r" );

#line 706 "cli.g"
        fclose (f);
      }
      else {zzFAIL(1,zzerr15,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd5, 0x1);
  }
}

void
#ifdef __USE_PROTOS
wcodes_cmd(void)
#else
wcodes_cmd()
#endif
{
#line 713 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 713 "cli.g"
  FILE *f=0; int flag = 1; char codes[8]; int ii=0;
#line 714 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    if ( (LA(1)==FROM) ) {
#line 714 "cli.g"
      zzmatch(FROM);
#line 714 "cli.g"
       f  = openfile( "r" );

    }
    else {
      if ( (LA(1)==BLANK) ) {
#line 714 "cli.g"
        zzmatch(BLANK);
#line 714 "cli.g"
        flag = 2;
        
      }
      else {
        if ( (setwd5[LA(1)]&0x2) ) {
        }
        else {zzFAIL(1,zzerr16,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
      }
    }
    zzEXIT(zztasp2);
    }
  }
#line 714 "cli.g"
  
  if ( f ) {
    if ( scodes_find_s1( f, 0, codes ) ) {
      scodes_push( codes );
      flag = 0;
    }
    fclose( f );
  }
  if( flag == 1 )
  mon_write_scode(sfd, 0);
  else if ( flag == 2 ) {
    mon_write_scode(sfd, -1);
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd5, 0x4);
  }
}

void
#ifdef __USE_PROTOS
serial_cmd(void)
#else
serial_cmd()
#endif
{
#line 733 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 733 "cli.g"
  char *cp=0; char *pn=0; int ii = 0; char buf[64];
  LOOK(1);
  if ( (LA(1)==STATUS) ) {
#line 734 "cli.g"
    zzmatch(STATUS);
#line 734 "cli.g"
    serial_status( sfd );
    
  }
  else {
    if ( (LA(1)==TDEBUG) ) {
#line 735 "cli.g"
      zzmatch(TDEBUG);
#line 735 "cli.g"
      {
        zzBLOCK(zztasp2);
        zzMake0;
        {
        LOOK(1);
        if ( (LA(1)==ON) ) {
#line 735 "cli.g"
          zzmatch(ON);
#line 735 "cli.g"
          dbgser=1;
          
        }
        else {
          if ( (LA(1)==OFF) ) {
#line 735 "cli.g"
            zzmatch(OFF);
#line 735 "cli.g"
            dbgser=0;
            
          }
          else {zzFAIL(1,zzerr17,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
        zzEXIT(zztasp2);
        }
      }
    }
    else {
      if ( (LA(1)==PORT) ) {
#line 736 "cli.g"
        zzmatch(PORT);
#line 736 "cli.g"
        {
          zzBLOCK(zztasp2);
          zzMake0;
          {
          LOOK(1);
          if ( (LA(1)==STR) ) {
#line 736 "cli.g"
             pn  = filename();

          }
          else {
            if ( (setwd5[LA(1)]&0x8) ) {
            }
            else {zzFAIL(1,zzerr18,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
          }
          zzEXIT(zztasp2);
          }
        }
#line 737 "cli.g"
        
        if ( pn ) {		// got a new port to use,
          // do a little error checking
          if ( *pn ) {
            cp = strdup( pn );	// must make a copy
            if ( !cp ) {
              eexit( errno );
            }
          }
          portname = cp;			// set global name
          sfd = serial_reopen( sfd, portname );
          if(pn)
          free( pn ); // eliminate mem leak
        }
        xprint( "%s\n", (sfd==-1)?"unknown":portname );
      }
      else {
        if ( (LA(1)==BAUDRATE) ) {
#line 753 "cli.g"
          zzmatch(BAUDRATE);
#line 753 "cli.g"
           ii  = integer();

#line 753 "cli.g"
          
          serial_setspeed( sfd, ii );
        }
        else {
          if ( (LA(1)==READ) ) {
#line 756 "cli.g"
            zzmatch(READ);
#line 756 "cli.g"
             ii  = integer();

#line 756 "cli.g"
            
            if( ii<64)
            serial_read( sfd, buf, ii );
          }
          else {zzFAIL(1,zzerr19,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
      }
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd5, 0x10);
  }
}

void
#ifdef __USE_PROTOS
debug_cmd(void)
#else
debug_cmd()
#endif
{
#line 762 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
  LOOK(1);
  if ( (LA(1)==ON) ) {
#line 763 "cli.g"
    zzmatch(ON);
#line 763 "cli.g"
    debug = 1;
    
  }
  else {
    if ( (LA(1)==OFF) ) {
#line 764 "cli.g"
      zzmatch(OFF);
#line 764 "cli.g"
      debug = 0;
      
    }
    else {
      if ( (LA(1)==SERIAL) ) {
#line 765 "cli.g"
        zzmatch(SERIAL);
#line 765 "cli.g"
        {
          zzBLOCK(zztasp2);
          zzMake0;
          {
          LOOK(1);
          if ( (LA(1)==ON) ) {
#line 765 "cli.g"
            zzmatch(ON);
#line 765 "cli.g"
            dbgser=1;
            
          }
          else {
            if ( (LA(1)==OFF) ) {
#line 765 "cli.g"
              zzmatch(OFF);
#line 765 "cli.g"
              dbgser=0;
              
            }
            else {zzFAIL(1,zzerr20,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
          }
          zzEXIT(zztasp2);
          }
        }
      }
      else {
        if ( (LA(1)==MONITOR) ) {
#line 766 "cli.g"
          zzmatch(MONITOR);
#line 766 "cli.g"
          {
            zzBLOCK(zztasp2);
            zzMake0;
            {
            LOOK(1);
            if ( (LA(1)==ON) ) {
#line 766 "cli.g"
              zzmatch(ON);
#line 766 "cli.g"
              dbgmon=1;
              
            }
            else {
              if ( (LA(1)==OFF) ) {
#line 766 "cli.g"
                zzmatch(OFF);
#line 766 "cli.g"
                dbgmon=0;
                
              }
              else {zzFAIL(1,zzerr21,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
            }
            zzEXIT(zztasp2);
            }
          }
        }
        else {
          if ( (LA(1)==FILEINFO) ) {
#line 767 "cli.g"
            zzmatch(FILEINFO);
#line 767 "cli.g"
            
            statinfo( fileno(infile) );
            xprint("\n");
            statinfo( fileno(outfile) );
            xprint("\n");
            
          }
          else {
            if ( (LA(1)==ALL) ) {
#line 773 "cli.g"
              zzmatch(ALL);
#line 774 "cli.g"
              {
                zzBLOCK(zztasp2);
                zzMake0;
                {
                LOOK(1);
                if ( (LA(1)==ON) ) {
#line 774 "cli.g"
                  zzmatch(ON);
#line 774 "cli.g"
                  debug = 1; dbgser=1; dbgmon=1;
                  
                }
                else {
                  if ( (LA(1)==OFF) ) {
#line 775 "cli.g"
                    zzmatch(OFF);
#line 775 "cli.g"
                    debug = 0; dbgser=0; dbgmon=0;
                    
                  }
                  else {zzFAIL(1,zzerr22,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
                }
                zzEXIT(zztasp2);
                }
              }
            }
            else {zzFAIL(1,zzerr23,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
          }
        }
      }
    }
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd5, 0x20);
  }
}

void
#ifdef __USE_PROTOS
scope_cmd(void)
#else
scope_cmd()
#endif
{
#line 781 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 782 "cli.g"
  zzmatch(ON);
#line 782 "cli.g"
  
#ifdef USE_SCOPE
  do_shell(scopecmd);
  //			do_scope();
#endif
  
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd5, 0x40);
  }
}

void
#ifdef __USE_PROTOS
set_cmd(void)
#else
set_cmd()
#endif
{
#line 791 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd5, 0x80);
  }
}

void
#ifdef __USE_PROTOS
version_cmd(void)
#else
version_cmd()
#endif
{
#line 795 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 796 "cli.g"
  xprint("%s\n", BUILD);
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd6, 0x1);
  }
}

void
#ifdef __USE_PROTOS
source_cmd(void)
#else
source_cmd()
#endif
{
#line 800 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 800 "cli.g"
  FILE *f=0;
#line 801 "cli.g"
   f  = openfile( "r" );

#line 801 "cli.g"
  
  if ( f ) {
    sourcing = 1;
    ANTLR( cmd_file(), f );
    sourcing = 0;
    fclose( f );
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd6, 0x2);
  }
}

void
#ifdef __USE_PROTOS
run_cmd(void)
#else
run_cmd()
#endif
{
#line 811 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 811 "cli.g"
  unsigned int pc,a,x,h ; struct HC08stack stack; pc=a=x=h=0;
#line 812 "cli.g"
   pc  = integer();

#line 812 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    if ( (setwd6[LA(1)]&0x4) ) {
#line 812 "cli.g"
       a  = integer();

#line 812 "cli.g"
      {
        zzBLOCK(zztasp3);
        zzMake0;
        {
        LOOK(1);
        if ( (setwd6[LA(1)]&0x8) ) {
#line 812 "cli.g"
           x  = integer();

#line 812 "cli.g"
          {
            zzBLOCK(zztasp4);
            zzMake0;
            {
            LOOK(1);
            if ( (setwd6[LA(1)]&0x10) ) {
#line 812 "cli.g"
               h  = integer();

            }
            else {
              if ( (setwd6[LA(1)]&0x20) ) {
              }
              else {zzFAIL(1,zzerr24,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
            }
            zzEXIT(zztasp4);
            }
          }
        }
        else {
          if ( (setwd6[LA(1)]&0x40) ) {
          }
          else {zzFAIL(1,zzerr25,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
        zzEXIT(zztasp3);
        }
      }
    }
    else {
      if ( (setwd6[LA(1)]&0x80) ) {
      }
      else {zzFAIL(1,zzerr26,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
    zzEXIT(zztasp2);
    }
  }
#line 812 "cli.g"
  
  stack.pc = pc;
  stack.a = a;
  stack.x = x;
  stack.h = h;
  mon_runstack( sfd, &stack);
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd7, 0x1);
  }
}

void
#ifdef __USE_PROTOS
cpu_cmd(void)
#else
cpu_cmd()
#endif
{
#line 823 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 824 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    if ( (setwd7[LA(1)]&0x2) ) {
#line 824 "cli.g"
      cpu_part();
    }
    else {
      if ( (setwd7[LA(1)]&0x4) ) {
      }
      else {zzFAIL(1,zzerr27,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
    zzEXIT(zztasp2);
    }
  }
#line 824 "cli.g"
  xprint( "%s\n", cpu_name( clicpu ) );
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd7, 0x8);
  }
}

void
#ifdef __USE_PROTOS
cpu_part(void)
#else
cpu_part()
#endif
{
#line 828 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 828 "cli.g"
  if( clicpu ) cpu_close( clicpu );
#line 830 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    if ( (LA(1)==GP32) ) {
#line 830 "cli.g"
      zzmatch(GP32);
#line 830 "cli.g"
      clicpu = cpu_open( "gp32" );
      
    }
    else {
      if ( (LA(1)==AB32) ) {
#line 831 "cli.g"
        zzmatch(AB32);
#line 831 "cli.g"
        clicpu = cpu_open( "ab32" );
        
      }
      else {
        if ( (LA(1)==JB8) ) {
#line 832 "cli.g"
          zzmatch(JB8);
#line 832 "cli.g"
          clicpu = cpu_open( "jb8"  );
          
        }
        else {zzFAIL(1,zzerr28,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
      }
    }
    zzEXIT(zztasp2);
    }
  }
#line 834 "cli.g"
  
  if( !clicpu )
  eprint( "open cpu failed\n");
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd7, 0x10);
  }
}

void
#ifdef __USE_PROTOS
send_cmd(int ecnt)
#else
send_cmd(ecnt)
 int ecnt ;
#endif
{
#line 843 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
#line 843 "cli.g"
  char ch[64], echo[2]; int i=0, j=0; unsigned int csum = 0;
#line 844 "cli.g"
  {
    zzBLOCK(zztasp2);
    int zzcnt=1;
    zzMake0;
    {
    do {
#line 844 "cli.g"
       ch[i]  = integer();

#line 844 "cli.g"
      if(i<64) i++;
      zzLOOP(zztasp2);
      LOOK(1);
    } while ( (setwd7[LA(1)]&0x20) );
    zzEXIT(zztasp2);
    }
  }
#line 844 "cli.g"
  
  while( j<i ) {
    serial_write( sfd, ch, i );
    if (  ecnt &&  ecnt<2  )
    serial_read( sfd, echo,  ecnt );
    csum += ch[j];
    j++;
  }
  dprint( "checksum = 0x%04X\n", csum&0xffff );
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd7, 0x40);
  }
}

void
#ifdef __USE_PROTOS
dtr_cmd(void)
#else
dtr_cmd()
#endif
{
#line 857 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  zzMake0;
  {
  LOOK(1);
  if ( (LA(1)==ON) ) {
#line 858 "cli.g"
    zzmatch(ON);
#line 858 "cli.g"
    dtr_on(sfd);;
    
  }
  else {
    if ( (LA(1)==OFF) ) {
#line 859 "cli.g"
      zzmatch(OFF);
#line 859 "cli.g"
      dtr_off(sfd);
      
    }
    else {zzFAIL(1,zzerr29,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
  }
  zzEXIT(zztasp1);
  return;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd7, 0x80);
  }
}

struct _rv30
#ifdef __USE_PROTOS
address_expression(void)
#else
address_expression()
#endif
{
  struct _rv30 _retv;
#line 864 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  PCCTS_PURIFY(_retv,sizeof(struct _rv30))
  zzMake0;
  {
#line 865 "cli.g"
  int x1=0; int x2=0; int op; _retv.base = 0; op = 0;
#line 866 "cli.g"
   x1  = integer();

#line 866 "cli.g"
  {
    zzBLOCK(zztasp2);
    zzMake0;
    {
    LOOK(1);
    if ( (setwd8[LA(1)]&0x1) ) {
#line 866 "cli.g"
      {
        zzBLOCK(zztasp3);
        zzMake0;
        {
        LOOK(1);
        if ( (LA(1)==PLUS) ) {
#line 866 "cli.g"
          zzmatch(PLUS);
#line 866 "cli.g"
          op=1;
          
        }
        else {
          if ( (LA(1)==COLON) ) {
#line 866 "cli.g"
            zzmatch(COLON);
#line 866 "cli.g"
            op=2;
            
          }
          else {zzFAIL(1,zzerr30,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
        }
        zzEXIT(zztasp3);
        }
      }
#line 866 "cli.g"
       x2  = integer();

    }
    else {
      if ( (setwd8[LA(1)]&0x2) ) {
      }
      else {zzFAIL(1,zzerr31,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
    }
    zzEXIT(zztasp2);
    }
  }
#line 866 "cli.g"
  
  _retv.base=x1;
  if ( op==1 )
  _retv.count = x2;
  else if ( op==2 )
  _retv.count = x2-x1+1;
  else
  _retv.count = 1;
  zzEXIT(zztasp1);
  return _retv;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd8, 0x4);
  return _retv;
  }
}

int
#ifdef __USE_PROTOS
integer(void)
#else
integer()
#endif
{
  int   _retv;
#line 878 "cli.g"
  zzRULE;
  Attrib i, j, h, ci;
  zzBLOCK(zztasp1);
  PCCTS_PURIFY(_retv,sizeof(int  ))
  zzMake0;
  {
  LOOK(1);
  if ( (LA(1)==INT) ) {
#line 879 "cli.g"
    zzmatch(INT);
    i = zzaCur;

#line 879 "cli.g"
    _retv= i.ival;
    
  }
  else {
    if ( (LA(1)==MINT) ) {
#line 880 "cli.g"
      zzmatch(MINT);
      j = zzaCur;

#line 880 "cli.g"
      _retv= j.ival;
      
    }
    else {
      if ( (LA(1)==MOTINT) ) {
#line 881 "cli.g"
        zzmatch(MOTINT);
        h = zzaCur;

#line 881 "cli.g"
        _retv =  h.ival;
        
      }
      else {
        if ( (LA(1)==CINT) ) {
#line 882 "cli.g"
          zzmatch(CINT);
          ci = zzaCur;

#line 882 "cli.g"
          _retv =  ci.ival;
          
        }
        else {zzFAIL(1,zzerr32,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
      }
    }
  }
  zzEXIT(zztasp1);
  return _retv;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd8, 0x8);
  return _retv;
  }
}

struct _rv32
#ifdef __USE_PROTOS
opt_address_expression(void)
#else
opt_address_expression()
#endif
{
  struct _rv32 _retv;
#line 886 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  PCCTS_PURIFY(_retv,sizeof(struct _rv32))
  zzMake0;
  {
#line 886 "cli.g"
  int a=-1,c=-1;
  LOOK(1);
  if ( (LA(1)==End) ) {
  }
  else {
    if ( (setwd8[LA(1)]&0x10) ) {
#line 887 "cli.g"
      { struct _rv30 _trv; _trv = address_expression();

      a = _trv.base; c  = _trv.count; }
#line 887 "cli.g"
      _retv.base = a; _retv.count=c;
    }
    else {zzFAIL(1,zzerr33,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
  }
  zzEXIT(zztasp1);
  return _retv;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd8, 0x20);
  return _retv;
  }
}

int
#ifdef __USE_PROTOS
opinteger(void)
#else
opinteger()
#endif
{
  int   _retv;
#line 891 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  PCCTS_PURIFY(_retv,sizeof(int  ))
  zzMake0;
  {
#line 891 "cli.g"
  int x; _retv=0;
  LOOK(1);
  if ( (LA(1)==End) ) {
  }
  else {
    if ( (setwd8[LA(1)]&0x40) ) {
#line 892 "cli.g"
       x  = integer();

#line 892 "cli.g"
      _retv = x;
    }
    else {zzFAIL(1,zzerr34,&zzMissSet,&zzMissText,&zzBadTok,&zzBadText,&zzErrk); goto fail;}
  }
  zzEXIT(zztasp1);
  return _retv;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd8, 0x80);
  return _retv;
  }
}

int
#ifdef __USE_PROTOS
scode_spec(char * buf)
#else
scode_spec(buf)
 char *buf ;
#endif
{
  int   _retv;
#line 897 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  PCCTS_PURIFY(_retv,sizeof(int  ))
  zzMake0;
  {
#line 897 "cli.g"
  unsigned char ch; int ii; for(ii=0;ii<SECCSIZE;ii++)  buf[ii]=0xff; ii = 0;
#line 898 "cli.g"
  {
    zzBLOCK(zztasp2);
    int zzcnt=1;
    zzMake0;
    {
    do {
#line 898 "cli.g"
       ch  = integer();

#line 898 "cli.g"
      
      if ( ii < SECCSIZE ) {
        buf[ii++] = ch;
      }
      zzLOOP(zztasp2);
      LOOK(1);
    } while ( (setwd9[LA(1)]&0x1) );
    zzEXIT(zztasp2);
    }
  }
  zzEXIT(zztasp1);
  return _retv;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd9, 0x2);
  return _retv;
  }
}

char *
#ifdef __USE_PROTOS
filename(void)
#else
filename()
#endif
{
  char *   _retv;
#line 908 "cli.g"
  zzRULE;
  Attrib s;
  zzBLOCK(zztasp1);
  PCCTS_PURIFY(_retv,sizeof(char *  ))
  zzMake0;
  {
#line 909 "cli.g"
  zzmatch(STR);
  s = zzaCur;

#line 909 "cli.g"
  _retv = trim( s.strdata);
  
  zzEXIT(zztasp1);
  return _retv;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd9, 0x4);
  return _retv;
  }
}

FILE *
#ifdef __USE_PROTOS
openfile(char * mode)
#else
openfile(mode)
 char *mode ;
#endif
{
  FILE *   _retv;
#line 915 "cli.g"
  zzRULE;
  zzBLOCK(zztasp1);
  PCCTS_PURIFY(_retv,sizeof(FILE *  ))
  zzMake0;
  {
#line 915 "cli.g"
  char *fn=0;
#line 916 "cli.g"
   fn  = filename();

#line 916 "cli.g"
  
  if ( *fn && fn ) {
    _retv = fopen(fn,  mode );
    if ( _retv == 0 ) {
      eprint( "open \"%s\" failed\n", fn );
    }
  }
  free(fn);	// eliminate mem leak
  zzEXIT(zztasp1);
  return _retv;
fail:
  zzEXIT(zztasp1);
  zzsyn(zzMissText, zzBadTok, (ANTLRChar *)"", zzMissSet, zzMissTok, zzErrk, zzBadText);
  zzresynch(setwd9, 0x8);
  return _retv;
  }
}
