// $Id: cli.g,v 1.13 2003/12/16 21:47:56 rwuest Exp $
/*
	SPGMR08 68HC08GP32 Loader
	Command line interface (and rc file/command file parser)
	Copyright (C) 2001  Sirius Engineering Company

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
*/

/* This is a PCCTS grammer file          */
/* and processing rc and command files   */
/* It may look like c, but it's not      */

// supported commands:
//
// scode <8byte_code>
// wcode
// read <address>
// write <address> <value>
// quit	| exit | q
// ! shell command line

// Integers may be in decimal, c format hex (0x...) or motorola format ($...)


#header <<

typedef union {
		long long ival;
		char *hseq;		/*	hex sequence data is malloc'ed to this ptr */
		char *strdata;
		char str[128];
	} Attrib;
void zzcr_attr(Attrib *attr, int type, char *text );
#define USER_ZZSYN
>>

<<
#include <stdio.h>
#include <termios.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <setjmp.h>
#ifdef __cpluspluss
extern "C" {
#endif
#include <readline/readline.h>
#include <readline/history.h>
#ifdef __cpluspluss
}
#endif


#include "spgmr.h"
#include "s1utils.h"
#include "cli.h"
#include "mongp32.h"
#include "lin_serial.h"
#include "seccodes.h"
#include "ops.h"

#ifndef DEBUG_READLINE
	#define DBGRDL(X)
#else
	#define DBGRDL(X) if(debug){X}
#endif

int dbgpar = 0;
#ifndef DEBUG_PARSER
	#define DBGPAR(X)
#else
	#define DBGPAR(X) X
#endif

struct sigaction cli_act;

static volatile sig_atomic_t done=0;				// flag to exit cli read loop
sigjmp_buf env;			// jump buffer to handle interrupted readline
int sourcing = 0;		// disbles quit command while sourcing

#ifdef USE_SCOPE
char *scopecmd = "xterm -e scope -b9600&";
#else
char *scopecmd = 0;
#endif

// current CPU
static CPU *clicpu=0;

// function defs
static int do_shell( char *cline );
static void noerr(const char *e);
static void cli_int(int i);	// cli interrupt handler
static void sf_int(int i);	// sourcing a file interrupt handler



void cli()
{
	char *rbuffer;
	static char *promptText;		// static to avoid gcc setjmp warning.  Bug??

	zzerr = noerr;

	cli_act.sa_handler = cli_int;
	cli_act.sa_flags = SA_RESTART;
	sigemptyset(&cli_act.sa_mask);
	sigaction(SIGINT,&cli_act,NULL);	// clean ^c exit

//	rl_catch_signals = 1;

	if ( isatty(fileno(infile)) | (interactive>1) ) {
		promptText = "spgmr: ";	
		rl_readline_name = "spgmr08";
		using_history();
		read_history( SPGMRHISTORY );	// read the past history, ignore errors

		do {
			rbuffer = 0;
			sigsetjmp(env, 1);
			rbuffer = readline(promptText);
			DBGRDL(dprint( "readline returned: %s\n", rbuffer?rbuffer:"(NULL)");)
			if ( rbuffer) {
				if ( rbuffer[0] ) {
					add_history( rbuffer );
					if ( rbuffer[0] == '!' ) {		// handle a shell right here
						do_shell( &rbuffer[1] );
					} else {
						ANTLRs( cmd_line(), rbuffer );
					}
				}
				free (rbuffer);
			} else {			// EOF on empty line
				done = 1;
			}
		} while ( !done );

		write_history( SPGMRHISTORY );
		xprint("Bye!\n" );
	} else {						// non-interactive server mode
		int ll;

		errfile = stdout;
		promptText = "spgmr:";	
		rbuffer = (char *)malloc(2048);
		interactive = 0;

		do {
			//int n;
			xprint( "%s\n", promptText );
#if 1
			if ( fgets(rbuffer, 2048, infile ) ){
#else
			if ( (n = read( STDIN_FILENO, rbuffer, 2048)) ) {
				dprint( "read: %d\n", n );
				if ( n == -1 ) {
					perror("read");
				}
				rbuffer[n] = '\0';
#endif
				ll = strlen( rbuffer );
				if ( rbuffer[ll-1] == '\n' )
					rbuffer[ll-1] = '\0';
				if (debug) {
					dprint( "fgets: %s\n", rbuffer);
				}
				if ( rbuffer[0] ) {
					if ( rbuffer[0] == '!' ) {		// handle a shell right here
						do_shell( &rbuffer[1] );
					} else {
						ANTLRs( cmd_line(), rbuffer );
					}
				}
			} else {			// EOF on empty line
				done = 1;
			}
		} while ( !done );
	}

}


int parse_line(char *cmd)		// externa/ interface to process a string
{
	return ANTLRs( cmd_line(), cmd );
}

int do_shell( char *cline )
{
	char *shell;

	if (cline) {
		while ( *cline && (*cline == ' ') )
			cline++;
		if ( *cline ) {
			system( cline );
		} else if ( isatty(1) ) {
			shell = getenv("SHELL");
			shell = shell?shell:(char *)"/bin/sh";
			xprint( "exit or end-of-file to return to spgmr\n" );
			system(shell);
		}
		return 0;
	} else {
		return -1;
	}
}

int do_scope( void )
{
	int pid;
	char *args[] = {
		"xterm",
		"-e",
		"scope",
		"-b9600",
		0
	};

	if ( (pid = fork()) == 0 ) {	// the child
		execvp( "xterm" , args );
	}

	return pid;

}

void cli_int(int i)	// cli interrupt handler
{
	if ( interactive ) {
//		rl_crlf();
		rl_reset_line_state();
	}
//	siglongjmp( env, 1 );
}

void sf_int(int i)	// sourcing a file interrupt handler
{
}

void noerr(const char *e)
{
}


void zzcr_attr(Attrib *attr, int type, char *text )
{
	char *t;
	t = text;
	switch( type )	{
	case INT:
		attr->ival = atoi(text);
		break;
	case MINT:
		attr->ival = atoi(text);
		break;
	case MOTINT:
		t++;	/* pass $ */
		attr->ival = (int)strtol(t, NULL, 16);
		break;
	case CINT:
		t+=2;	/* pass 0x */
		attr->ival = (int)strtol(t, NULL, 16);
		break;
//	case SNAME:
//		t++; /* skip leading '{'  */
//		strncpy(attr->str,t,127);
//		attr->str[strlen(attr->str) - 1] = '\0';	/* skip trailing '}' */
//		break;
	case STR:
		//printf("STR %s\n", text);
		/* this is definitely a memory leak */
		/* should be no big deal unless you use a lot of these */
		/* or free the space when done with it */
		/* Strip leading spaces */
		while( *t && *t==' ' )
			t++;
		/* maybe strip the quote marks */
		if ( *t =='\"' ) {
			t++;
			text[ strlen(text)-1 ] = '\0';
		}
		attr->strdata = strdup( t );
		break;
 	default:
		attr->ival = atoi(text);
		break;
	}
}

/* modified error reporting function */
/*error reporting is partly broken - this needs fixin' */
#define WSYNBUFSIZE	 1024
void zzsyn(char *text, int tok, char *egroup, SetWordType *eset, int etok, int k, char *bad_text)
{
	char work[WSYNBUFSIZE];	// let's hope an error message is never bigger than this
						// need to do an atomic all to eprint
	int place = 0;
	int left = WSYNBUFSIZE;
	
    zzSyntaxErrCount++;                             /* MR11 */
	place = snprintf( work, WSYNBUFSIZE-place, "line %d: syntax error at \"%s\"", zzline, (tok==zzEOF_TOKEN)?"EOF":bad_text);
	if ( !etok && !eset ) {
		eprint( "%s\n", work );
		return;
	}
	if ( k==1 ) {
		place += snprintf( &work[place], WSYNBUFSIZE-place, " missing" );
	} else {
		place += snprintf( &work[place], WSYNBUFSIZE-place, "; \"%s\" not", bad_text );
		if ( zzset_deg(eset)>1 )
			place += snprintf( &work[place], WSYNBUFSIZE-place, " in" );
	}
	if ( zzset_deg(eset)>0 )
//		place += myzzedecode(eset, &work[place]);
		zzedecode(eset);
	else
		place += snprintf( &work[place], WSYNBUFSIZE-place, " %s", zztokens[etok] );
	if ( strlen(egroup) > 0 )
		place += snprintf( &work[place], WSYNBUFSIZE-place, " in %s", egroup);

	eprint("%s\n", work);
}

/*   END OF C CODE   */
>>

/****************************************************************************/
/*                 Tokens Definitions                                       */
/****************************************************************************/
#lexclass FileName
#token STR		"~[\n\r;]*"	<< zzmode(START); >>

#lexclass START
#token End		"@"
#token			"[\ \t\r]"	<<zzskip();>>
#token			"\n"		<<zzline++;>>
#token PLUS		"\+"
#token COLON	":"
//#token MINUS	"\-"
#token TO		"to"		<< zzmode(FileName); >>
#token FROM		"from"		<< zzmode(FileName); >>
#token ON		"on"
#token OFF		"off"

//#token QSTR		"\"~[\"\n]*\""
//#token SNAME	"\{[._A-Za-z][$.\-\@_A-Za-z0-9/]*\}"

// Commands
#token READ 	"read"
#token WRITE	"write"
#token ERASE	"erase"
#token PROGRAM	"program"
#token VERIFY	"verify"
#token QUIT		"(quit)|(exit)|(q)"
#token RESET	"reset"
#token SCODES	"scodes"
#token WCODES	"wcodes"
#token TDEBUG	"debug"
#token SERIAL	"serial"
#token SET		"set"
#token VERSN	"version"
#token SOURCE	"source"		<< zzmode(FileName); >>
#token RUN		"run"
#token COMMENT	"#~[\n]*"
#token SCOPE	"scope"
#token TCPU		"cpu"
// undocumented commands
#token SEND		"send"
#token SENDE1	"sende1"
#token SENDE2	"sende2"
#token DTR		"dtr"

// modifiers
#token MONITOR	"monitor"
#token STATUS	"status"
#token PORT		"port"			<< zzmode(FileName); >>
#token SP		"sp"
#token STACK	"stack"
#token FILEINFO	"fileinfo"
#token BLANK	"blank"
#token ALL		"all"
#token SHOW		"show"
#token PUSH		"push"
#token TRY		"try"
#token AS		"as"
#token THELP	"help"
#token BAUDRATE	"(baud)|(baudrate)|(speed)"

//recognized cpus:
#token AB32		"ab32"
#token GP32		"gp32"
#token JB8		"jb8"

// numbers
#token CINT		"0[xX][0-9a-fA-F]+"
#token INT		"[0-9]+"
#token MINT		"\-[0-9]+"
#token MOTINT	"$[0-9a-fA-F]+"

//#token STR		"\"~[\"\n]*\""
//#token SNAME	"\{[._A-Za-z][$.\-\@_A-Za-z0-9/]*\}"

#token GARBAGE "~[\ \n;\+:]*"

#errclass "Command" { READ WRITE ERASE PROGRAM VERIFY QUIT RESET SCODES
                      WCODES TDEBUG SERIAL COMMENT SCOPE SET VERSN SOURCE
                      RUN SEND SENDE1 SENDE2 TCPU
                    }
#errclass "Integer" { MINT CINT INT MOTINT }

/****************************************************************************/
/*                 The Grammer                                              */
/****************************************************************************/


// parse a file stream
cmd_file:
	( {cmd_linei} "\n" )* End
	;


// parse a null terminated buffer
cmd_line:
	cmd_linei End
	;


cmd_linei:
	command (";" {command})*
	;


command:
		READ	read_cmd
	|	WRITE	write_cmd
	|	ERASE	erase_cmd
	|	PROGRAM	program_cmd
	|	VERIFY	verify_cmd
	|	QUIT	quit_cmd
	|	RESET	reset_cmd
	|	SCODES	scodes_cmd
	|	WCODES	wcodes_cmd
	|	TDEBUG	debug_cmd
	|	SERIAL	serial_cmd
	|	COMMENT
	|	SCOPE	scope_cmd
	|	SET		set_cmd
	|	VERSN	version_cmd
	|	SOURCE	source_cmd
	|	RUN		run_cmd
	|	TCPU	cpu_cmd
	|	SEND	send_cmd[0]
	|	SENDE1	send_cmd[1]
	|	SENDE2	send_cmd[2]
	|	DTR		dtr_cmd
	;


read_cmd: << int a1,a2; char *buf=0; FILE *f=0; struct HC08stack stack; >>
		( address_expression > [a1,a2] { TO openfile["w"] > [f] } ) <<
			DBGPAR ( if(dbgpar) dprint( "read 0x%X+0x%X\n", a1, a2 ); )
			if ( a2>0 ) {
				buf = (char *)malloc(a2);
			}

			if ( a2>0 )	{
				mon_blkread( sfd, buf, a1, a2);
			}
	
			if ( !f ) {
				format_dump( buf, a1, a2 );
			} else {						// TO file
				fprints0( f, "SPGMR");
				fprints1( f, buf, a1, a2 );
				fprints9( f, 0 );
				fclose( f );
			}
			if (buf) {
				free(buf);
			}
		>>
	|	SP <<
			buf = (char *)malloc(2);
			mon_readsp(sfd, buf);
			xprint( "SP = 0x%02X%02X\n", buf[0], buf[1] );
			free( buf );
		>>
	| STACK <<
		mon_show_stack( sfd, &stack);
	>>
	;

/* This rule would require an ll(2) parser :(  */
/* maybe a better syntax for writing data */
//write_cmd: << int a1,c1,v; int a2,c2; unsigned char *buf; char *fn = 0; unsigned long s1address; int s1_len; >>
//	WRITE (
//		opt_address_expression > [a1,c1] (
//				integer > [v]  << printf("add val\n"); >>
//			|  FROM filename > [fn] { address_expression > [a2,c2] } << printf( "add from file add\n");	>>
//		)
//
//
//	)
//	;

write_cmd: << int a1,cnt,v; char *buf; char *fn=0; unsigned long address; int s1_len; >>	
		address_expression > [a1,cnt] (
			integer > [v] <<
				cnt = 1;
				if ( v>0xff) {
					cnt = 2;
				}
				buf = (char *)malloc(cnt);
				buf[cnt-1] = v;
				if (cnt==2)
					buf[0] = (v\>\>8) & 0xff;
				mon_blkwrite( sfd, buf, a1, cnt );
				free(buf);
			>>
		)
	| 	FROM filename > [fn] << if(fn) { mon_write_file( fn );  free(fn); }>>
	;


erase_cmd: << int x,a,c; int bflag = 0;>>	
	{address_expression > [a,c] << bflag = 1;>>}	<<
		if ( !clicpu ) {
			eprint( "Must set cpu\n");
		} else {
			if ( !bflag ) {
				mass_erase(clicpu);
			}
		}
	>>
	;


program_cmd: << int x,a,c; int bflag = 0; char *fn=0; >>	
	{address_expression > [a,c] << bflag = 1;>>}	FROM filename > [fn] <<
		if ( !clicpu ) {
			eprint( "Must set cpu\n");
		} else {
			if ( !bflag && fn && *fn ) {
				program_flash(clicpu,fn);
			}
		}
		if(fn)
			free(fn);
	>>
	;


verify_cmd: << int x,a,c; int bflag = 0; char *fn=0; >>	
	FROM filename > [fn] <<
		if ( !clicpu ) {
			eprint( "Must set cpu\n");
		} else {
			if (fn && *fn) {
				verify_flash(clicpu,fn);
			}
		}
		if(fn)
			free(fn);
	>>
	;


quit_cmd:
		<<
			if ( !sourcing )
				done = 1;
		>>
	;


reset_cmd:
		<<
			mon_reset(sfd);
		>>
	;



scodes_cmd:
		SEND scodes_send_
	|	SET  scodes_set_
	|	SHOW scodes_show_
	|	PUSH scodes_push_
//	|	TRY << clicpu >>?[eprint( "Must set cpu\n");] scodes_try_	// no good
//	|	TRY << if (!clicpu) {eprint( "Must set cpu\n"); zzconsumeUntilToken(End);} >> scodes_try_	// no workie
	|	TRY scodes_try_
	;

//scodes send from filename
//scodes send as int int int int int int int int
//scodes send int
//scodes send blank
scodes_send_: << FILE *f=0; int cnt; char buf[SECCSIZE]; >>
		FROM openfile["r"] > [f] <<
			if( f ) {
				cnt = scodes_find_s1( f, 0, buf );
				fclose( f );
				if (cnt == 0 ) {
					scodes_push( buf );
					mon_write_scode(sfd, 0);
					scodes_print(0);
				}
			}
		>>
	|	AS scode_spec[buf] > [cnt] << scodes_push( buf ); mon_write_scode(sfd, 0); >>
	|	integer > [cnt] <<
			if (cnt>=0 && cnt<SECCMAX )
				mon_write_scode(sfd, cnt);
		>>
	|	BLANK << mon_write_scode(sfd, -1);	>>
	;

//scodes set from file name
//scodes set int as int int int int int int int int
scodes_set_: << FILE *f=0; char *fn=0; int cnt; int which = 0; char buf[8]; >>
	FROM openfile["r"] > [f] <<
		if ( f ) {
			cnt = scodes_find_s1( f, 0, buf );
			fclose( f );
			if (cnt == 0 ) {
				scodes_push( buf );
				scodes_print(0);
			}
		}
		>>
	|	integer > [which] AS scode_spec[buf] > [cnt] << scodes_set( buf , which); >>
	;


//scodes show all
//scodes show int
//scodes show from filename
scodes_show_: << FILE *f=0; int cnt; char buf[SECCSIZE]; >>
		FROM openfile["r"] > [f] <<
			if ( f ) {
				cnt = scodes_find_s1( f, 0, buf );
				fclose( f );
				if (cnt == 0 ) {
					int ii;
					for( ii=0; ii<SECCSIZE; ii++ )
						xprint( "%02X ", buf[ii] );
					xprint("\n");
				}
			}
		>>
	|	integer > [cnt] << scodes_print(cnt); >>
	|	ALL <<
			for(cnt=0;cnt<SECCMAX;cnt++)
				scodes_print(cnt);
		>>
	;

		
//scodes push int as int int int int int int int int
//scodes push from filename
scodes_push_: << FILE *f=0; int cnt; char buf[SECCSIZE]; >>
		FROM openfile["r"] > [f] <<
			if ( f ) {
				cnt = scodes_find_s1( f, 0, buf );
				fclose( f );
				if (cnt == 0 ) {
					int ii;
					for( ii=0; ii<SECCSIZE; ii++ )
						xprint( "%02X ", buf[ii] );
					xprint("\n");
				}
			}
		>>
	|	integer > [cnt] << scodes_print(cnt); >>
	|	ALL <<
			for(cnt=0;cnt<SECCMAX;cnt++)
				scodes_print(cnt);
		>>
	;

// scodes try from filename
// scodes try int
// scodes try all
// scodes try blank
scodes_try_: << FILE *f=0; int cnt; unsigned char buf[8]; >>
		( integer > [cnt] | BLANK <<cnt = -1;>> ) <<
			if (!clicpu)
				eprint( "Must set cpu\n");
			else {
				cnt = reset_cpu( clicpu, cnt );
				xprint( cnt ? "failed\n" : "passed\n" );
			}
		>>
	|	ALL
	|	FROM openfile["r"] > [f] << fclose (f); >>
	;





wcodes_cmd: << FILE *f=0; int flag = 1; char codes[8]; int ii=0;>>
	{ FROM openfile["r"] > [f] | BLANK <<flag = 2; >> } <<
		if ( f ) {
			if ( scodes_find_s1( f, 0, codes ) ) {
				scodes_push( codes );
				flag = 0;
			}
			fclose( f );
		}
		if( flag == 1 )
			mon_write_scode(sfd, 0);
		else if ( flag == 2 ) {
			mon_write_scode(sfd, -1);
		}
	>>
	;




serial_cmd: << char *cp=0; char *pn=0; int ii = 0; char buf[64];>>
		STATUS << serial_status( sfd ); >>
	|	TDEBUG ( ON <<dbgser=1;>> | OFF <<dbgser=0;>> )
	|	PORT { filename > [pn] }
		<<
			if ( pn ) {		// got a new port to use,
							// do a little error checking
				if ( *pn ) {
					cp = strdup( pn );	// must make a copy
					if ( !cp ) {
						eexit( errno );
					}
				}
				portname = cp;			// set global name
				sfd = serial_reopen( sfd, portname );
				if(pn)
					free( pn ); // eliminate mem leak
			}
			xprint( "%s\n", (sfd==-1)?"unknown":portname );
		>>
	|	BAUDRATE integer > [ii] <<
			serial_setspeed( sfd, ii );
		>>
	| READ integer > [ii] <<
			if( ii<64)
				serial_read( sfd, buf, ii );
		>>
	;

debug_cmd:
		ON << debug = 1; >>
	|	OFF <<debug = 0;  >>
	|	SERIAL ( ON <<dbgser=1;>> | OFF <<dbgser=0;>> )
	| 	MONITOR  ( ON <<dbgmon=1;>> | OFF <<dbgmon=0;>> )
	|   FILEINFO  <<
			statinfo( fileno(infile) );
			xprint("\n");
			statinfo( fileno(outfile) );
			xprint("\n");
		>>
	|	ALL	(
			ON << debug = 1; dbgser=1; dbgmon=1; >>
		|	OFF<< debug = 0; dbgser=0; dbgmon=0; >>
		)
	;



scope_cmd:	
		ON <<
		#ifdef USE_SCOPE
			do_shell(scopecmd);
//			do_scope();
		#endif
	>>
	;


set_cmd:
	;


version_cmd:
	<< xprint("%s\n", BUILD); >>
	;


source_cmd:	<< FILE *f=0; >>
	openfile["r"] > [f] <<
		if ( f ) {
			sourcing = 1;
			ANTLR( cmd_file(), f );
			sourcing = 0;
			fclose( f );
		}
	>>
	;
// run pc {A { X {H}}}
run_cmd: << unsigned int pc,a,x,h ; struct HC08stack stack; pc=a=x=h=0; >>
	integer > [pc] { integer > [a] { integer > [x] {integer > [h] } } } <<
		stack.pc = pc;
		stack.a = a;
		stack.x = x;
		stack.h = h;
		mon_runstack( sfd, &stack);
	>>
	;


// manage CPU
cpu_cmd:
		{ cpu_part } << xprint( "%s\n", cpu_name( clicpu ) ); >>
	;


cpu_part: << if( clicpu ) cpu_close( clicpu ); >>
	(
			GP32 << clicpu = cpu_open( "gp32" ); >>
		|	AB32 << clicpu = cpu_open( "ab32" ); >>
		|	JB8  << clicpu = cpu_open( "jb8"  ); >>
	)
	<<
		if( !clicpu )
			eprint( "open cpu failed\n");
	>>
	;

		
// These are for debugging
// Be careful, there's no bounds checking here
send_cmd[int ecnt]: << char ch[64], echo[2]; int i=0, j=0; unsigned int csum = 0; >>
	(integer > [ch[i]] << if(i<64) i++;>> )+ <<
		while( j<i ) {
			serial_write( sfd, ch, i );
			if ( $ecnt && $ecnt<2  )
				serial_read( sfd, echo, $ecnt );
			csum += ch[j];
			j++;
		}
		dprint( "checksum = 0x%04X\n", csum&0xffff );
	>>
	;
	

dtr_cmd:
		ON	<<dtr_on(sfd);;>>
	|	OFF	<<dtr_off(sfd);>>
	;

	
	
address_expression > [int base, int count]:
	<< int x1=0; int x2=0; int op; $base = 0; op = 0;>>
		integer > [x1] { (PLUS <<op=1;>> | COLON <<op=2;>>) integer > [x2] } <<
			$base=x1;
			if ( op==1 )
				$count = x2;
			else if ( op==2 )
				$count = x2-x1+1;
			else
				$count = 1;
		>>
	;


integer > [int r]:
		i:INT	<<$r=$i.ival;>>
	|	j:MINT	<<$r=$j.ival;>>
	|	h:MOTINT	<<$r = $h.ival; >>
	|	ci:CINT		<<$r = $ci.ival; >>
	;


opt_address_expression > [int base, int count]: << int a=-1,c=-1; >>
	| address_expression > [a,c] << $base = a; $count=c; >>
	;


opinteger > [int r]: <<int x; $r=0; >>
	| integer > [x] <<$r = x;>>
	;


	
scode_spec[char *buf] > [int cnt]: << unsigned char ch; int ii; for(ii=0;ii<SECCSIZE;ii++) $buf[ii]=0xff; ii = 0;>>
	( integer > [ch] <<
			if ( ii < SECCSIZE ) {
				$buf[ii++] = ch;
			}
		>>
	)+
	;
	

// files and filenames
filename >  [char *fn]:
	s:STR	<< $fn = trim($s.strdata); >>
	;

// this rule take a filename, opens it in the passed mode and free's the name space
// On success, the FILE * is returned
// On failure - print an error message and return NULL
openfile[char *mode] > [FILE *f]: << char *fn=0; >>
	filename > [fn] <<
		if ( *fn && fn ) {
			$f = fopen(fn, $mode );
			if ( $f == 0 ) {
				eprint( "open \"%s\" failed\n", fn );
			}
		}
		free(fn);	// eliminate mem leak
	>>
	;


// $Log: cli.g,v $
// Revision 1.13  2003/12/16 21:47:56  rwuest
// patches from Anders Nystrom <andesr@southpole.se>
// add jb8 processor
// add patch target
// fix default port bug
// added load_module()
//
// Revision 1.12  2003/12/16 21:21:54  rwuest
// changed unsigned chars to chars
// made c++ happy
// removed myzzedecode
//
// Revision 1.11  2001/08/17 11:54:43  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.10  2001/08/11 21:13:06  rwuest
// lots of extra parens removed.
// added serial read int command - not tested
//
// Revision 1.9  2001/08/11 14:27:52  rwuest
// Added "serial speed nnn" command
//
// Revision 1.8  2001/08/04 16:42:23  rwuest
// Most cpu specific operations now use the CPU * type through the ops abstaction
// rewrote scodes, all security ops are here now.
//  scodes (set|try|show|push|send)
// added openfile rule to open a file from a passed filename
// moved command name tokens up to the command rule
// cleaned out some cruft
// added a scode_spec to accept 8 integers and put them in a buffer
//
// Revision 1.7  2001/07/29 03:23:46  rwuest
// Added debug all (on|off) - sets all debug flags
// interrupt done flag is now static volatile sig_atomic_t
// added catchall token ("~[
// ;+:]*") at end of token defs
// add SA_RESTART to sigint flags
// playing with a do_scope function that does the fork/exec thing
// Don't ry to reselt the input line when not interactive
// Started adding a wcodes int int int int int int int int command (not
// finished)
// Added debug all on/off
// "do shell" calls the fork/exec thing now
//
// Revision 1.6  2001/07/27 16:44:24  rwuest
// uniform error print statements
// fileinfo through xprint
//
// Revision 1.5  2001/07/26 21:19:52  rwuest
// Fixed scope command
// Added source command
// Cleaned up start rule names and grammer to better handle lines and streams.
// Added rule for filename to better handle ';' seperated commands
// Every filename reference to s:STR is now to the filename rule
// Add source, run, send, sende1, sende2 commands
// added FROM filename clause to wcodes
// disable quit while sourceing a file
// erase command now calls erase
// Added read to file and read stack
// renamed all the command rules to have _cmd as suffix
//
// Revision 1.4  2001/07/22 02:41:30  rwuest
// disable scope if USE_SCOPE isn't defined
// can't get the command to pass from make so for now
// user must edit cli.g and rebuild. bummer
//
// Revision 1.3  2001/07/22 02:19:54  rwuest
// explict 0 out the interactive flag in server mode
// Add version command
//
