; $Id: erase_gp32.s,v 1.2 2001/08/17 11:54:43 rwuest Exp $
;
; SPGMR08 68HC08GP32 Loader
; Erase flash memory
; Copyright (C) 2001  Sirius Engineering Company
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License version 2.
; See the file COPYING that is distributed with this program.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; Mass erase flash memory
; per algorithm:
; Motorola doc. no. MC68HC908GP32/H  REV. 4, pg 168
; Cpu speed = 2.476 MHZ

	cpu 68HC08
	include gp32

	org $100
	
t4ms equ 40
; erase
; A determines mass or block
; A = 0 is mass erase
; A = 1 is block erase of block specified by address in H:X

erase:
	cmp #0
	beq mass_erase
	cmp #1
	beq block_erase
block_erase:	
	swi			; invalid - exit	
mass_erase:
	lda #$06
	sta FLCR	; step 1, set ERASE and MASS bits
	lda FLBPR	; step 2, Read the FLBPR
	sta >FLBPR	; step 3, (per *) Write FLBPR
	lda #8
	dbnza	*	; step 4, 3*8 cycle delay, delay 10 us
	lda #$e	
	sta	FLCR	; step 5, set HVEN
	lda	#t4ms
	bsr delay	; step 6, wait 4 ms
	lda #$c
	sta FLCR	; step 7, clear ERASE bit
	lda #1
	bsr delay	; step 8, wait 100 us
	clra
	sta	FLCR 	; step 9, Clear HVEN
	swi
	
; Delay 100us
; Delay times required are 10 us, 100 us and 4 mS
; 10 us is 25 cycles
; 100 us is 247 cycles
; 4 ms is 9904 cycles
t100 equ 80
delay:
	pshh		; 2
l1:	ldx	#t100	; 2
l2:	dbnzx	l2	; 3
	dbnza	l1
	pulh		; 2
	rts			; 4
	
	end erase

	
