; SPGMR08 68HC08GP32 Loader
; jb8 erase/program/verify FLASH memory
; Copyright (C) 2002, Anders Nystrom
;
;  This program is free software; you can redistribute it and/or modify
;  it under the terms of the GNU General Public License version 2.
;  See the file COPYING that is distributed with this program.
;
;  This program is distributed in the hope that it will be useful,
;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;  GNU General Public License for more details.
;
; CPU speed 3 MHz
;

	cpu 68HC08
VERIFY	EQU $fc03
ERASE	EQU $fc06
PROGRAM	EQU $fc09
FLBPR	EQU $fe09


	org $48

ctrlbyt:rmb 1
cpuspd:	rmb 1
laddr:	rmb 2
databuf:rmb 64


	org $90

; Mass/block erase
;
; Input
; A	   Mass or block erase
;	   Mass erase if 128
;	   Block erase if 0
; HX	   Contains any address in the range to be erased
erase:
	bit #$7f
	bne er0
	lsra
	sta ctrlbyt
	jsr unprotect
	jsr ERASE
er0:	swi


	org $100

; Program/verify FLASH
;
; Input
; A	   Size of range to be programmed/verified
;	   Program if possitive
;	   Verify if negative
; HX	   FLASH start address to be verified
; databuf  Contains data to be programmed/verified
;
; Output
; C bit	   Carry bit set if verify passes
; databut  Contains the data in the range of FLASH memory
prog_vrfy:
	clr ctrlbyt
	tsta
	bpl addr
	beq die
	and #$7f
	inc ctrlbyt

addr:	deca
	sthx laddr
	add laddr+1
	sta laddr+1
	clra
	adc laddr
	sta laddr
	lda ctrlbyt
	cbeqa #0,prog
	cbeqa #1,vrfy
die:	swi
	
vrfy:	jsr VERIFY
	swi
prog:	bsr unprotect
	jsr PROGRAM
	swi


; Unprotect FLASH
;
; Input
; H	   High byte of address in block to be erased/programmed
unprotect:
	pshh
	pula
	cmp #$ff
	beq unp0
	add #2
unp0:	sta FLBPR
	mov #4*3,cpuspd
	rts
