// $Id: main.c,v 1.10 2003/12/16 21:25:39 rwuest Exp $
/*
	SPGMR08 68HC08GP32 Loader
	Copyright (C) 2001  Sirius Engineering Company

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <signal.h>
#include <sys/wait.h>
//#include <proplist.h>

#include "spgmr.h"
#include "s1utils.h"
#include "lin_serial.h"
#include "mongp32.h"
#include "cli.h"
#include "seccodes.h"
#include "ops.h"

int dbg = 0;

#ifndef DEBUG
	#define DBG(X)
#else
	#define DBG(X) X
#endif

int sfd = -1;
void usr1_handler(int i);
void usr2_handler(int i);
void child_handler(int i);

int main( int argc, char *argv[] )
{
	struct sigaction act;
	unsigned char c[256];
	int add;

	act.sa_handler = iexit;
	act.sa_flags = 0;
	sigemptyset(&act.sa_mask);
//	sigaction(SIGINT,&act,NULL);	// clean ^c exit
	sigaction(SIGHUP,&act,NULL);	// sig from a dying shell - exit graceful
	sigaction(SIGTERM,&act,NULL);
	
	act.sa_handler = usr1_handler;
	act.sa_flags = SA_RESTART;
	sigaction(SIGUSR1,&act,NULL);

	act.sa_handler = usr2_handler;
	act.sa_flags = SA_RESTART;
	sigaction(SIGUSR2,&act,NULL);

	act.sa_handler = child_handler;
	act.sa_flags = SA_RESTART;
	sigaction(SIGCHLD,&act,NULL);

	process_args(argc, argv);
	scodes_init();
	ops_init();

	sfd = serial_open( portname );
//	serial_setspeed( sfd, 9600 );

	DBG( if (dbg) dprint( "Open serial as fd = %d\n", sfd ); )

	if ( interactive) {
		cli();
	}
	else {
/*
From design.txt:
	The order in which command line actions are taken is important.  The order
	on the command line itself is not important.  If an erase option is given,
	the chip will first be erased.  If a program option is given, the part
	will then be programmed.  If an read option is given, the part will be
	read.  Finally, if the run opttion is given, it will be processed.  So it
	is prossible to run the program with a command like this:
*/
		if (eflag) {	// do erase
		}

		if (pflag) {	// program part
		}

		if (rflag) {	// read part
		}

		if (Rflag) {	// run
		}


	}
	if(debug)
		dprint("Out main\n");
	eexit(0);
	return 0;	// never gets here but makes gcc happy
}

/****************************************************************************/
/*                 void eexit(int e)                                        */
/*                                                                          */
/* error program exit - do any required cleanup, but don't block            */
/****************************************************************************/
void eexit(int e)
{
	scodes_close();
	serial_close(sfd);
	if(debug)
		dprint ("eexit(%d)\n",e);
	if (sfd != -1 ) {
		serial_close( sfd );
	}
	exit(e);
}

/****************************************************************************/
/*                 void iexit(int i)                                        */
/*                                                                          */
/* interrupted program exit - do any required cleanup, but don't block      */
/****************************************************************************/
void iexit(int i)
{
	if(debug)
		dprint ("exit on interrupt %d\n",i);
	eexit(0);
}

void usr1_handler(int i)
{
	// just throw some data to the standard output for testing the pipes
	write( STDOUT_FILENO, "this is a test\nspgmr:\n", 22 );
	return;
}

void usr2_handler(int i)
{
	if (debug ) {
		debug = 0;
		dbgser = 0;
		dbgmon = 0;
	} else {
		debug = 1;
		dbgser = 1;
		dbgmon = 1;
	}
}

void child_handler(int i)
{
	int pid, status, serrno;
	serrno = errno;
	while (1)
	{
		pid = waitpid (WAIT_ANY, &status, WNOHANG);
		if (pid < 0)
		{
			perror ("waitpid");
			break;
		}
		if (pid == 0)
			break;
		//notice_termination (pid, status);
	}
	errno = serrno;

}
// $Log: main.c,v $
// Revision 1.10  2003/12/16 21:25:39  rwuest
// no fd to cli()
//
// Revision 1.9  2001/08/17 11:54:42  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.8  2001/08/11 14:34:32  rwuest
// toggle all debug flags with SIGUSR2
// baudrate set in serial_open, don;t call it here.
//
// Revision 1.7  2001/08/04 17:20:02  rwuest
// added scodes_close() and serial_close() calls at exit
// cleaned out some debug cruft
// explicitly set baudrate after serial_open()
//
// Revision 1.6  2001/07/29 03:27:51  rwuest
// added sigusr2 handler to enable all debug outputs
// added sigchld handler to pick up a dying scope program
// removed SA_RESETHAND flag for sigusr2
//
// Revision 1.5  2001/07/27 16:45:50  rwuest
// debug stuff through dprintf
// no more fileinfo on debug startup
// throw out some cruft
//
// Revision 1.4  2001/07/26 21:05:18  rwuest
// new mon_write_scode() needs an index arg.
// include stdlib.h
//
// Revision 1.3  2001/07/22 02:00:38  rwuest
// remove libPropList
//
// Revision 1.2  2001/07/22 00:33:00  rwuest
// No significant functional changes here, just debugging pipes.
// Added SIGTERM to exit with text.
// Added SIGUSR1 handler to spit out some stuff and a prompt (for
// pipe sync debugging with tkspgmr)
// prints out lots of file info on startup - more debug text
// Debug messages now all print through dprint()
// Standard out now all goes througn xprint()
// Error messages all go through eprint()
// added log messages at end
//

