// $Id: mongp32.c,v 1.9 2003/12/16 21:47:56 rwuest Exp $
/*
    SPGMR08 68HC08GP32 Loader
    gp32 monitor functions
    Copyright (C) 2000  Robert Wuest (rwuest@wuest.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <endian.h>
#include <errno.h>
#include <string.h>

#include "spgmr.h"
#include "lin_serial.h"
#include "seccodes.h"
#include "mongp32.h"
#include "s1utils.h"

int dbgmon = 0;

#ifndef DEBUG_MONITOR
	#define DBGMON(X)
#else
	#define DBGMON(X) X
#endif

/*****************************************/
/* BIG FAT WARNINGs:                     */
/* Endianness is INTEL                   */
/* and int is 32 bits                    */
/* developed on                          */
/* i686-pc-linux-gnu                     */
/* Not thread safe                       */
/*****************************************/

union {
	unsigned int i;
	char bytes[4];
} eu;

#ifdef __LITTLE_ENDIAN
#define hbyte eu.bytes[1]
#define lbyte eu.bytes[0]
#endif

// need these functions:
// mon_write_e();	// write and get an echo
// mon write_e2();	// write and get 2 back

/****************************************************************************/
/*                 int mon_read(int fd, char *buf, int address)             */
/*                                                                          */
/* Read memory (code 0x4A)                                                  */
/* Read a byte from target at memory address and store it in buf.           */
/* Return 0, on success, -1 on failure.                                     */
/****************************************************************************/
int mon_read(int fd, char *buf, int address)
{
	char code = 0x4A;
	char rdc[2] = {0,0};
	int cnt;

	if( !buf )			// sanity check
		return -1;

	DBGMON( if(dbgmon) dprint( "mon_read( address = 0x%04X )", address ); )

	eu.i = address;
	serial_iflush(fd);
	serial_write( fd, &code, 1 );
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;
	
	serial_write( fd, &hbyte, 1 );
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;

	serial_write( fd, &lbyte, 1 );
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;
	
	cnt = serial_read( fd, rdc, 1 );


err_exit:
	if (cnt == 0) {
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	} else {
		*buf = rdc[0];
		DBGMON( if(dbgmon) dprint( " = 0\n" ); )
		return 0;
	}
}


/****************************************************************************/
/*                 int mon_iread(int fd, char *buf)                         */
/*                                                                          */
/* Indexed Read memory (code 0x1a)                                          */
/* Read two byte from target at last accessed memory address and return it. */
/* Return 0 on success,  -1 on failure.                                     */
/****************************************************************************/
int mon_iread(int fd, char *buf)
{
	char code = 0x1A;
	char rdc[2] = {0,0};
	int cnt;

	DBGMON( if(dbgmon) dprint( "mon_iread()" ); )

	serial_iflush(fd);
	serial_write( fd, &code, 1 );
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;
	
	cnt = serial_read( fd, rdc, 2 );

err_exit:
	if (cnt == 2 ){
		*buf++ = rdc[0];
		*buf = rdc[1];
		DBGMON( if(dbgmon) dprint( " = 0\n" ); )
		return 0;
	} else {
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	}
}


/****************************************************************************/
/*                 int mon_write(int fd, char *buf, int address)            */
/*                                                                          */
/* Write memory (code 0x49)                                                 */
/* Write byte at buf to target at memory address.                           */
/* Return 0 on success, -1 on failure.                                      */
/****************************************************************************/
int mon_write(int fd, char *buf, int address)
{
	char code = 0x49;
	char rdc[2] = {0,0};
	int cnt;

	if( !buf )			// sanity check
		return -1;

	DBGMON( if(dbgmon) dprint( "mon_write( address = 0x%04X)", address ); )

	eu.i = address;
	serial_iflush(fd);
	serial_write( fd, &code, 1 );
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;
	
	serial_write( fd, &hbyte, 1 );		// address high
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;

	serial_write( fd, &lbyte, 1 );		// address low
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;
	
	cnt = serial_write( fd, buf, 1 );	// data
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed


err_exit:
	if (cnt != 2) {
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	} else {
		DBGMON( if(dbgmon) dprint( " = 0\n"); )
		return 0;
	}
}


/****************************************************************************/
/*                 int mon_iwrite(int fd, char *buf)                        */
/*                                                                          */
/* Indexed Write memory (code 0x19)                                         */
/* Write byte in buf to target last accessed memory address.                */
/* Return 0 on success, -1 on failure.                                      */
/****************************************************************************/
int mon_iwrite(int fd, char *buf)
{
	char code = 0x19;
	char rdc[2] = {0,0};
	int cnt;

	DBGMON( if(dbgmon) dprint( "mon_iwrite()" ); )

	serial_iflush(fd);
	serial_write( fd, &code, 1 );		// send code
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;
	
	serial_write( fd, buf, 1 );			// send data
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed

err_exit:	
	if (cnt == 2 ) {
		DBGMON( if(dbgmon) dprint( " = 0\n" ); )
		return 0;
	} else {
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	}
}


/****************************************************************************/
/*                 int mon_readsp(int fd, char *buf)                        */
/*                                                                          */
/* Read stack pointer (code 0x0c)                                           */
/* Reads stack pointer to buf                                               */
/* Return 0 on success, -1 on failure.                                      */
/****************************************************************************/
int mon_readsp(int fd, char *buf)
{
	char code = 0x0c;
	char rdc[2] = {0,0};
	int cnt;

	DBGMON( if(dbgmon) dprint( "mon_readsp()\n" ); )

	serial_iflush(fd);
	serial_write( fd, &code, 1 );		// send read stack pointer code
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	if(cnt!=2) goto err_exit;
	
	cnt = serial_read( fd, rdc, 2 );	// get stack pointer

err_exit:	
	if ( cnt == 2  ){
		*buf++ = rdc[0];
		*buf = rdc[1];
		DBGMON( if(dbgmon) dprint( " = 0\n" ); )
		return 0;
	} else {
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	}
}


/****************************************************************************/
/*                 int mon_run(int fd)                                      */
/*                                                                          */
/* Run user program (code 0x28)                                             */
/* Monitor will execute PULH and RTI                                        */
/* Return 0 on success, -1 on failure                                       */
/****************************************************************************/
int mon_run(int fd)
{
	char code = 0x28;
	char rdc[2] = {0,0};
	int cnt;

	DBGMON( if(dbgmon) dprint( "mon_run()" ); )

	serial_iflush(fd);
	serial_write( fd, &code, 1 );		// send run code
	serial_owait(fd);
	cnt = serial_read( fd, rdc, 2 );	// echoed
	
	if ( cnt == 2 ){
		DBGMON( if(dbgmon) dprint( " = 0\n" ); )
		return 0;
	} else {
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	}
}

/****************************************************************************/
/*                 int mon_reset(int fd)                                    */
/*                                                                          */
/* Reset the target by taking DTR false, slep a bit, then back true         */
/* Return 0 on success, -1 on failure                                       */
/****************************************************************************/
int mon_reset(int fd)
{
	char c;
	int cnt;
	
	DBGMON( if(dbgmon) dprint( "mon_reset()" ); )

	serial_iflush(fd);
	
	if ( dtr_off(fd) ) {
		return -1;
	}

	usleep(200000L);	// sleep 200 ms
	
	if ( dtr_on(fd) ) {
		return -1;
	}

	usleep(100000L);	// sleep 100 ms
	cnt  = serial_read( fd, &c, 1);
	if ( cnt == 1 ) {
		DBGMON( if(dbgmon) dprint( " = 0\n" ); )
		return 0;
	} else {
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	}
}


/****************************************************************************/
/*               int mon_blkread(int fd, int address, char *buf, int count) */
/*                                                                          */
/* Read count bytes from target at memory address and store them in buf.    */
/* Return 0 on success, Return -1 on failure.                               */
/****************************************************************************/
int mon_blkread(int fd, char *buf, int address, int count)
{
	int ii,oe;
	int place = 0;

	// there are 4 sets of count value types:
	// 0 - return 0 and do nothing
	// 1 - just call mon_read
	// even - count greater than 1
	// odd  - count greater than 1

	if ( !buf )						// sanity
		return -1;

	if ( count == 0 ) {				// 0 - return 0 and do nothing
		return 0;
	} else {
		oe = count % 2;
		if ( !oe ) {				// even has no remainder
			ii = mon_read( fd, &buf[place++], address );
			if ( ii == -1 ) {
				return -1;
			} else {
				count --;
				address++;
			}
		}
		ii = mon_read( fd, &buf[place++], address );	// read at least 1
		if ( ii == -1 ) {
			return -1;
		} else {
			count --;
			address++;
		}
		while ( count > 0 ) {		// now an even number to fetch
			ii = mon_iread( fd, &buf[place] );
			place +=2;
			count -= 2;
		}
	}

	return 0;
}


/****************************************************************************/
/*               int mon_blkwrite(int fd, int address, char *buf, int count)*/
/*                                                                          */
/* Read count bytes from target at memory address and store them in buf.    */
/* Return 0 on success, Return -1 on failure.                               */
/****************************************************************************/
int mon_blkwrite(int fd, char *buf, int address, int count)
{
	int ii;
	int place = 0;

	if ( !buf )						// sanity
		return -1;
	
	if ( count == 0 ) {				// 0 - return 0 and do nothing
		return 0;
	} else {
		ii = mon_write( fd, &buf[place++], address );	// first byte & sets address
		if ( ii == -1 ) {
			return -1;
		} else {
			count--;
		}
		while ( count > 0 ) {				// write the rest
			ii = mon_iwrite( fd, &buf[place] );
			place++;
			count--;
		}
	}

	return 0;
}


int mon_write_scode( int fd, int w ) // note P&E read $FEFB first ??????
{
	int ii,cnt;
	char rdc[2];
	char buf[SECCSIZE];
	
	DBGMON( if(dbgmon) dprint( "mon_write_scode()" ); )

	if ( w == -1 ) {
		// send 0xff's if which is -1
		memset( buf, 0xFF, SECCSIZE );
	} else {
		// else get the codes
		if (scodes_get( buf, w ) < 0)
			return -1;
	}

	// send buf out serial
	serial_iflush(fd);

	for ( ii=0; ii<8; ii++) {
		serial_write( fd, &buf[ii], 1 );
		serial_owait(fd);
		cnt = serial_read( fd, rdc, 2 );	// echoed
		if (cnt!=2) goto err_exit;
	}
	cnt = serial_read( fd, rdc, 1 );	// break is sent

	if ( cnt == 1 ) {
		DBGMON( if(dbgmon) dprint( " = 0\n" ); )
		return 0;
	} else {
err_exit:
		DBGMON( if(dbgmon) dprint( " = -1\n" ); )
		return -1;
	}

}


/****************************************************************************/
/*                 int mon_read_stack( int fd, struct HC08stack *stack)     */
/*                                                                          */
/* Read the stack to HC08stack at stack                                     */
/* returns 0 on success, -1 on error                                        */
/****************************************************************************/
int mon_read_stack( int fd, struct HC08stack *stack)
{
	char buf[6];
	unsigned int a;

	mon_readsp( fd, buf);
	a = buf[0]*256+buf[1];
	stack->sp = a;
	
	mon_blkread( sfd, buf, a, 6);
	stack->h = buf[0];
	stack->cc = buf[1];
	stack->a = buf[2];
	stack->x = buf[3];
	stack->pc = buf[4]*256 + buf[5];
	return 0;
}

/****************************************************************************/
/*                 int mon_show_stack( int fd, struct HC08stack *stack)     */
/*                                                                          */
/* Read the stack to HC08stack at stack and print it                        */
/* returns 0 on success                                                     */
/****************************************************************************/
int mon_show_stack( int fd, struct HC08stack *stack)
{
	int rv;

	rv = mon_read_stack(sfd, stack);
	if ( !rv ) {
		dprint( "SP = 0x%04X\n", stack->sp );
		dprint( "H  = 0x%02X\n", stack->h );
		dprint( "CC = 0x%02X\n", stack->cc );
		dprint( "A  = 0x%02X\n", stack->a );
		dprint( "X  = 0x%02X\n", stack->x );
		dprint( "PC = 0x%04X\n", stack->pc );
	}
	return rv;
}


/****************************************************************************/
/*                 int mon_read_ax( int fd, struct HC08stack *stack)        */
/*                                                                          */
/* Read a, cc and x only from the target's stack to HC08stack at stack      */
/* returns 0 on success                                                     */
/****************************************************************************/
int mon_read_ax( int fd, struct HC08stack *stack)
{
	char buf[3];
	unsigned int a;

	if (!mon_readsp( fd, buf) ) {
		a = buf[0]*256+buf[1];
		stack->sp = a;
		mon_blkread( sfd, buf, a+2, 2);
		stack->a = buf[0];
		stack->x = buf[1];
		return 0;
	}
	return -1;
}
/****************************************************************************/
/*               int mon_runpc(int fd, unsigned int pc)                     */
/*                                                                          */
/* Read SP, setup PC for RUN and call mon_run                               */
/*                                                                          */
/* Stack looks like this:                                                   */
/* SP+0:                                                                    */
/* SP+1: H	     <- value returned by readsp                                */
/* SP+2: CC                                                                 */
/* SP+3: A                                                                  */
/* SP+4: X                                                                  */
/* SP+5: PCH                                                                */
/* SP+6: PCL                                                                */
/* SP+7:                                                                    */
/* Returns 0 on success                                                     */
/****************************************************************************/
int mon_runpc(int fd, unsigned int pc)
{
	char buf[7];
	char spb[2];
	int sp;
	int cnt;

	mon_readsp( fd, spb );
	sp = spb[0]*256+spb[1];
	
	buf[0] = (pc>>8)&0xff;
	buf[1] = pc&0xff;
	mon_write( fd, &buf[0], sp+4);
	mon_write( fd, &buf[1], sp+5);
	
	mon_run(fd);

	return 0;
}


/****************************************************************************/
/*               int mon_runstack(int fd, unsigned int pc)                  */
/*                                                                          */
/* Read SP, setup stack for RUN and call mon_run                            */
/* Returns 0 on success                                                     */
/****************************************************************************/
int mon_runstack(int fd, struct HC08stack *stack)
{
	char buf[7];
	char spb[2];
	int sp_plus;
	int cnt;

	mon_readsp( fd, spb );
	sp_plus = spb[0]*256+spb[1];

	mon_write( fd, &(stack->h), sp_plus);
	mon_iwrite( fd, &(stack->cc) );
	mon_iwrite( fd, &(stack->a) );
	mon_iwrite( fd, &(stack->x) );
	
	buf[0] = (stack->pc>>8)&0xff;
	buf[1] = stack->pc&0xff;
	mon_iwrite( fd, &buf[0] );
	mon_iwrite( fd, &buf[1] );

	mon_run(fd);

	return 0;
}

/****************************************************************************/
/*               int mon_write_file(char *filename)                         */
/*                                                                          */
/* Copies S19 file to target RAM                                            */
/* FIXME: need to validate destination addresses                            */
/* Returns 0 on success                                                     */
/****************************************************************************/
int mon_write_file(char *filename)
{
	char *buf;
	FILE *fwork;
	unsigned long address;
	int s1_len;
	int rv = 0;

	buf = (char *)malloc(MAXS1DATA);
	if ( !buf ) {
		eprint("memory error\n");
		return(-1);
	}
	fwork = fopen(filename, "r" );
	if ( fwork == 0 ) {
		eprint( "open \"%s\" failed\n", filename );
		return(errno);
	}
	
	while( s1ffillBuffer(
				fwork,	 /* where to get the data */
				buf,	 /* where to put data */
				&address,	  /* return start address here */
				&s1_len,	/* return actual databuffer byte count here */
				LCS_NORMAL) && !rv )
		rv = mon_blkwrite( sfd, buf, address, s1_len	);

	fclose( fwork );
	free( buf);
	
	return rv;
}

/****************************************************************************/
/*               int mon_write_file_range(char *filename, unsigned int dest,*/
/*                                   unsigned int start, unsigned int count)*/
/*                                                                          */
/* Copies count bytes from address start in S19 file to target RAM at dest  */
/* FIXME: need to validate destination addresses                            */
/* Returns 0 on success                                                     */
/****************************************************************************/
int mon_write_file_range(char *filename, unsigned int dest, unsigned int start, unsigned int count)
{
	char *buf;
	FILE *fwork;
	unsigned long address;
	int s1_len;

	if( count <= 0 )
		return -1;

	buf = (char *)malloc(count);
	if ( !buf ) {
		eprint("memory error\n");
		return(-1);
	}
	fwork = fopen(filename, "r" );
	if ( fwork == 0 ) {
		eprint( "open \"%s\" failed\n", filename );
		return(errno);
	}
	s1fsearchBLk(fwork, buf, start, (start+count-1) );

	fclose( fwork );
	free( buf);
	
	return 0;

}

// $Log: mongp32.c,v $
// Revision 1.9  2003/12/16 21:47:56  rwuest
// patches from Anders Nystrom <andesr@southpole.se>
// add jb8 processor
// add patch target
// fix default port bug
// added load_module()
//
// Revision 1.8  2003/12/16 21:19:31  rwuest
// changed unsigned chars to chars
//
// Revision 1.7  2001/08/17 11:54:43  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.6  2001/08/11 21:14:29  rwuest
// increased reset dtr_off time to 200 ms
//
// Revision 1.5  2001/08/04 16:47:00  rwuest
// better error handling
// show stack is a debug operation, out though dprint
// use memset for 0xff filled buffers.
// don't operate directly on the sec_codes array - that's all in the sec codes functions now
//
// Revision 1.4  2001/07/29 03:29:21  rwuest
// changed reset dtr times to 100 ms, seems ok.
//
// Revision 1.3  2001/07/26 21:01:51  rwuest
// mon_write_scode() get data from gloabl  array now.
// Wiil write ff's if which = -1.
// added int mon_read_stack(), int mon_show_stack(), int mon_read_ax()
// int mon_runpc(), int mon_runstack(), int mon_write_file(),
// int mon_write_file_range()
//
// Revision 1.2  2001/07/22 00:29:55  rwuest
// Debug messages now all print through dprint()
// Standard out now all goes througn xprint()
// Error messages all go through eprint()
// added log messages at end
//

