// $Id: ops.c,v 1.5 2003/12/16 21:47:56 rwuest Exp $
/*
	SPGMR08 68HC08GP32 Loader
	Program/erase Operations
	Copyright (C) 2001  Sirius Engineering Company

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
*/

/*
Global Functions here:
	mass_erase()
	block_erase()
	program_flash()
	declared in spgmr08.h
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "spgmr.h"
#include "mongp32.h"
#include "lin_serial.h"
#include "ops.h"


/****************************************************************************/
/*            struct Cpus                                                   */
/*                                                                          */
/* Holds the names of known processors and how to initialze one             */
/* To add a new cpu, give it any valid name here, whish is pretty much      */
/* anything that is not a script token like a keyword or number.            */
/* Write the init function and point to it here.                            */
/****************************************************************************/
struct Cpus {
	char *name;
	CPU *(*init_function)(CPU *ops);
} cpus[] = {
	{ "gp32", gp32_init},
	{ "ab32", ab32_init},
	{ "jb8",  jb8_init},
	{ NULL, NULL }
};


/****************************************************************************/
/*                 CPU *cpu_open(char *name)                                */
/*                                                                          */
/* Search the known cpus, and if the name is found, allocate a new CPU,     */
/* call its init function and return the pointer to it.                     */
/* Returns 0 on any error.                                                  */
/****************************************************************************/
CPU *cpu_open(char *name)
{
	int ii;
	CPU *tcpu;

	if ( !name ) {
		return 0;
	}
	
	if ( name ) {
		for ( ii=0; cpus[ii].name; ii++ ) {
			if ( !strcmp( name, cpus[ii].name) ) {
				if ( (tcpu = (CPU *)malloc( sizeof(CPU))) != 0 ) {
					if ( !(cpus[ii].init_function(tcpu) ) ) {
						free( tcpu );
					} else {
						return tcpu;
					}
				}
			}
		}
	}
	return 0;
}

/****************************************************************************/
/*                 int cpu_close(CPU *xcpu)                                 */
/*                                                                          */
/* Close an open CPU, deallocate its memory                                 */
/* return 0 on success, -1 for failure                                      */
/****************************************************************************/
int cpu_close(CPU *xcpu)
{
	if ( xcpu )	{
		if (xcpu->cpu_data)
			free(xcpu->cpu_data);
		free( xcpu );
		return 0;
	} else {
		return -1;
	}
}

const char *cpu_name( CPU *xcpu )
{
	if (!xcpu) {
		return "None";
	} else {
		return xcpu->cpu_name;
	}
}

int ops_init(void)
{
	return 0;
}

int reset_cpu( CPU *xcpu, int which )
{
	if( ! xcpu )
		return -1;
	return xcpu->reset_function( xcpu, which );

}

int mass_erase(CPU *xcpu )
{
	if( ! xcpu )
		return -1;
	return xcpu->erase_function(xcpu);

}
	
int program_flash( CPU *xcpu, char *filename )
{
	if( ! xcpu )
		return -1;
	return xcpu->flash_function( xcpu, filename );
}

int program_eeprom( CPU *xcpu, char *filename )
{
	if( ! xcpu )
		return -1;
	return xcpu->eeprom_function( xcpu, filename );
}


int verify_flash( CPU *xcpu, char *filename )
{
	if( ! xcpu )
		return -1;
	return xcpu->verify_function( xcpu, filename );
}


int verify_eeprom( CPU *xcpu, char *filename )
{
	if( ! xcpu )
		return -1;
	return xcpu->verify_function( xcpu, filename );
}

char *processor_module(char *x)
{
	return NULL;
}


int load_module(CPU *xcpu, char *mod)
{
	int rv = 0;

	if (xcpu->loaded_module != mod) {
		xcpu->loaded_module = NULL;
		xprint("Loading module %s\n", mod);
		if (mon_write_file(mod) == 0) {
			xcpu->loaded_module = mod;
		} else {
			eprint("module load error\n");
			rv = -1;
		}
	}
	return rv;
}

