// $Id: ops_gp32.c,v 1.9 2003/12/16 21:47:56 rwuest Exp $
/*
	SPGMR08 68HC08GP32 Loader
	Program/erase Operations
	Copyright (C) 2001  Sirius Engineering Company

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
*/

/*
Global Functions here:
	mass_erase_gp32()
	block_erase_gp32()
	program_flash_gp32()
	declared in spgmr08.h
*/

#include <stdio.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <stdlib.h>
#include <sys/time.h>

#include "spgmr.h"
#include "mongp32.h"
#include "lin_serial.h"
#include "s1utils.h"
#include "seccodes.h"
#include "ops.h"
#include  "timec.h"

struct sigaction old_act;
struct sigaction ops_act;
void setup_int(void);
void ops_int(int i);	// interrupt handler
void restore_int(void);	// restore to original handler
static volatile sig_atomic_t aborted;				// interrupted flag
static struct timeval starttv;
static struct timeval nowtv;
double secs1,secs2;


//static char *erase_module_gp32 = "./erase_gp32.s19";
//static char *program_module_gp32 = "./program_gp32.s19";
//static char *loaded_module = 0;

struct hc908 {
	unsigned long flash_base;
	unsigned long flash_end;
	unsigned long vector_base;
	unsigned long vector_end;
	unsigned long eeprom_base;
	unsigned long eeprom_end;
};

/* startup init code */
CPU *gp32_init( CPU *xcpu)
{
	struct hc908 *cpu_data;

	xcpu->cpu_name="gp32";
	xcpu->erase_function = mass_erase_gp32;
	xcpu->flash_function = program_flash_gp32;
	xcpu->verify_function = verify_flash_gp32;
	xcpu->eeprom_function = NULL;
	xcpu->reset_function = reset_gp32;
	xcpu->run_function = run_gp32;
	xcpu->read_function = NULL;
	xcpu->write_function = NULL;
	xcpu->run_function = run_gp32;
	xcpu->eeprom_module =NULL;
	xcpu->verify_module = LIBDIR"/program_gp32.s19";
	xcpu->program_module = LIBDIR"/program_gp32.s19";
	xcpu->erase_module  = LIBDIR"/erase_gp32.s19";
	xcpu->loaded_module = 0;
	cpu_data = (struct hc908 *)malloc( sizeof(struct hc908) );
	xcpu->cpu_data = cpu_data;
	if ( cpu_data ) {
		cpu_data->flash_base  = 0x8000;
		cpu_data->flash_end   = 0xDFFF;
		cpu_data->vector_base = 0xFFDC;
		cpu_data->vector_end  = 0xFFFF;
		cpu_data->eeprom_base = 0;
		cpu_data->eeprom_end  = 0;
	}
	
	return xcpu;
}

/* startup init code */
CPU *ab32_init( CPU *xcpu)
{
	struct hc908 *cpu_data;

	xcpu->cpu_name="ab32";
	xcpu->erase_function = mass_erase_gp32;
	xcpu->flash_function = program_flash_gp32;
	xcpu->verify_function = verify_flash_gp32;
	xcpu->eeprom_function = NULL;
	xcpu->reset_function = reset_ab32;
	xcpu->run_function = run_gp32;
	xcpu->read_function = NULL;
	xcpu->write_function = NULL;
	xcpu->eeprom_module =NULL;
	xcpu->verify_module = LIBDIR"/program_ab32.s19";
	xcpu->program_module = LIBDIR"/program_ab32.s19";
	xcpu->erase_module  = LIBDIR"/erase_gp32.s19";
	xcpu->loaded_module = 0;
	cpu_data = (struct hc908 *)malloc( sizeof(struct hc908) );
	xcpu->cpu_data = cpu_data;
	if ( cpu_data ) {
		cpu_data->flash_base  = 0x8000;
		cpu_data->flash_end   = 0xDFFF;
		cpu_data->vector_base = 0xFFD0;
		cpu_data->vector_end  = 0xFFFF;
		cpu_data->eeprom_base = 0x0800;
		cpu_data->eeprom_end  = 0x09FF;
	}
	return xcpu;
}


/****************************************************************************/
/*               int mass_erase_gp32(void)                                  */
/*                                                                          */
/* Perform a mass erase on a GP32 chip                                      */
/* Returns 0 on success                                                     */
/****************************************************************************/
int mass_erase_gp32(CPU *xcpu)
{
	char * t_module;
	struct HC08stack stack;
	char c;

	if (load_module(xcpu, xcpu->erase_module) != 0)
		return -1;

	stack.pc = 0x100;
	stack.a = 0;	// mass erase
	mon_runstack( sfd, &stack);	// start loader
	serial_read( sfd, &c, 1 );
	xprint( "Erased\n" );
	return 0;
}

/****************************************************************************/
/*               int program_or_verify_flash_gp32(char *filename)            */
/*                                                                          */
/* Programs filename to a GP32 chip                                         */
/* Returns 0 on success                                                     */
/****************************************************************************/
int program_or_verify_flash_gp32( CPU *xcpu, char *filename, int op )
{
	char *t_module;
	int ii, jj, cnt;
	char buf[64], echo[64];
	unsigned long address;
	int s1_len;
	FILE *fwork;
	struct HC08stack stack;
	unsigned int csum, tcsum;
	int ssize =1;
	int rsize = 64;
	int pcount=0;	// progress counter
	int rv;		// return value
	int retry_cnt = 0;	// retry counter on data load error
	struct hc908 *cpu_data;
	int vector_cnt;

	if( op != 0 && op != 128 )
		return -1;

	aborted = 0;
	setup_int();
	gettimeofday( &starttv, NULL);			// save now time; start time
	secs1 = tvtod( &starttv );
	
	cpu_data = ((struct hc908 *)(xcpu->cpu_data));
	
	fwork = fopen(filename, "r" );
	if ( fwork == 0 ) {
		restore_int();
		eprint( "open \"%s\" failed\n", filename );
		return(errno);
	}
	rv = load_module(xcpu, xcpu->program_module);

	for ( ii=cpu_data->flash_base; ii<=cpu_data->flash_end && !aborted && !rv ; ii+=rsize ) {
		retry_cnt = 0;
		for(jj=0; jj<rsize; jj++)
			buf[jj] = 0xff;
		cnt = s1fsearchBLk(fwork, buf, ii, ii+rsize-1 );
		csum = 0;
		for(jj=0; jj<rsize; jj++)
			csum += buf[jj];
		csum &= 0xffff;
		if ( cnt ) {
			gettimeofday( &nowtv, NULL);
			secs2 = tvtod( &nowtv );

			xprint( "0x%04X  %d  %-04.3f\n", ii, cnt, secs2-secs1);
			secs1 = secs2;
//			fflush( stderr );
//			if (++pcount>3) {		// this makes 4 address / output line
//				xprint("\n");
//				pcount = 0;
//			}
retry:
			stack.pc = 0x100;
			stack.a = 0;	// send data code
			stack.x = rsize;
			mon_runstack( sfd, &stack);	// start loader

			ssize = 64;
			for( jj=0; jj<rsize; jj+=ssize ) {	// send row of data
//				serial_baud( sfd, B19200);
				serial_iflush( sfd );
				serial_write( sfd, &buf[jj], ssize );
				serial_read( sfd, &echo[jj], ssize );
//				serial_baud( sfd, B9600);
			}
			
//			dprint( "checksum = 0x%04X\n", csum );
			mon_read_ax(sfd, &stack);
			tcsum = stack.a*256+stack.x;
			if ( tcsum != csum ) {				// not valid checksum
				if ( !retry_cnt ) {
					retry_cnt ++;
					xprint( "retry 0x%04X  %d\n", ii, cnt);
					goto retry;
				} else {
					eprint("data load error\n");
					mon_show_stack( sfd, &stack );
					xcpu->loaded_module = 0;
					rv = -1;
				}
			} else {							// valid checksum					
				// do the programming/verifying
				stack.pc = 0x100;
				stack.a = cnt+op;			// program, set count
											// verify, sets count +128
				stack.h = (ii>>8) & 0xff;	// high byte destination address
				stack.x = ii& 0xff;			// low byte of destination
				mon_runstack( sfd, &stack);	// start program
				serial_read( sfd, echo, 1 );
				mon_read_ax( sfd, &stack);	// get result
				if( stack.a != 0 ) {		// verification failed
					eprint(op?"verify error at 0x%02X%02X\n":"program error at 0x%02X%02X\n", stack.h, stack.x);
					mon_show_stack( sfd, &stack );
					xcpu->loaded_module = 0;
					rv = -1;
				}
			}
		}
	}

	// now do the vector space
	for(jj=0; jj<rsize; jj++)
		buf[jj] = 0xff;
	cnt = s1fsearchBLk(fwork, buf, cpu_data->vector_base, cpu_data->vector_end );
	vector_cnt = (cpu_data->vector_end - cpu_data->vector_base) + 1;
	if ( cnt != vector_cnt ) {
		wprint( "you need to define all your vectors\n" );
	}
	csum = 0;
	for(jj=0; jj<vector_cnt; jj++)
		csum += buf[jj];
	csum &=0xffff;
	if ( cnt && !rv && !aborted ) {
		xprint( "0x%04X  %d\n", cpu_data->vector_base, vector_cnt);
		stack.pc = 0x100;
		stack.a = 0;	// send data code
		stack.x = vector_cnt;
		mon_runstack( sfd, &stack);	// start loader
		ssize = vector_cnt;
		for( jj=0; jj<vector_cnt; jj+=ssize ) {	// send row of data
//			serial_baud( sfd, B19200);
			serial_iflush( sfd );
			serial_write( sfd, &buf[jj], ssize );
			serial_read( sfd, &echo[jj], ssize );
//			serial_baud( sfd, B9600);
		}
//		dprint( "checksum = 0x%04X\n", csum );
		mon_read_ax(sfd, &stack);
		tcsum = stack.a*256+stack.x;
		if ( tcsum != csum ) {				// valid?
			eprint("load error\n");
			mon_show_stack( sfd, &stack );
			xcpu->loaded_module = 0;
			rv = -1;
		} else {
		
			// do the programming/verifying
			stack.pc = 0x100;
			stack.a = cnt+op;			// program, set count
										// verify, set count +128
			stack.h = (cpu_data->vector_base>>8) & 0xFF;	// high byte destination address
			stack.x = cpu_data->vector_base & 0xFF;			// low byte of destination
	
			mon_runstack( sfd, &stack);	// start program
			serial_read( sfd, echo, 1 );	// get the 0 at the end, will wait VTIME
			mon_read_ax(sfd, &stack);	// get result
			if( stack.a != 0 ) {		// verification failed
				eprint(op?"verify error at 0x%02X%02X\n":"program error at 0x%02X%02X\n", stack.h, stack.x);
				mon_show_stack( sfd, &stack );
				xcpu->loaded_module = 0;
				rv = -1;
			} else {
				// we want $FFF6$FFFD
				// buf is at cpu_data->vector_base
				// $FFF6-$FFDC=$1A
				scodes_push( &buf[0XFFF6-cpu_data->vector_base] );
			}
		}
	}

	restore_int();
	fclose( fwork );
	if (aborted) {
		xprint ("Aborted\n" );
	} else if (rv !=-1) {
		xprint(op?"Verify OK\n":"Program OK\n");
	} else {
		xprint(op?"Verify failed\n":"Program failed\n");
	}
	return 0;
}

int program_flash_gp32( CPU *xcpu, char *filename )
{

	return program_or_verify_flash_gp32( xcpu,filename, 0 );
}

int verify_flash_gp32( CPU *xcpu, char *filename )
{

	return program_or_verify_flash_gp32( xcpu, filename, 128 );
}

/****************************************************************************/
/*               int reset_gp32(CPU *cpu, int which)                        */
/*                                                                          */
/* Reset and send security codes.                                           */
/* Returns 0 on success, ie, the security codes sent and accepted           */
/****************************************************************************/
int reset_gp32(CPU * xcpu, int which)
{
	int ii;
	char c;
	if ( which>=0 && which<SECCMAX ) {
		mon_reset( sfd );
		mon_write_scode(sfd, which);

/* Read location $0040 and return bit 6's value                             */
		ii = mon_read( sfd, &c, 0x0040);
		if ( c & 0x40 )		// test bit 6
			return 0;
	}
	return -1;
}


/****************************************************************************/
/*               int reset_gp32(CPU *cpu, int which)                        */
/*                                                                          */
/* Reset and send security codes.                                           */
/* Returns 0 on success                                                     */
/****************************************************************************/
int reset_ab32(CPU * xcpu, int which)
{
	int ii;
	char c;

	if ( which>=-1 && which<SECCMAX ) {
		mon_reset( sfd );
		if (mon_write_scode(sfd, which)) goto err_exit;
/* Read location $0050 and return bit 6's value                             */
		ii = mon_read( sfd, &c, 0x0050);
		if ( c & 0x40 )		// test bit 6
			return 0;
	}
err_exit:
	return -1;
}

/****************************************************************************/
/*               int run_gp32(CPU *cpu, int pc)                             */
/*                                                                          */
/* Runs at pc.                                                              */
/* There are some special PC values.  Any value in the vector space will    */
/* essentially do an indirect run, ie., fetch the vector and run there.     */
/* Returns 0 on success                                                     */
/****************************************************************************/
int run_gp32(CPU *xcpu, int pc)
{
	return 0;
}

void setup_int(void)
{
	ops_act.sa_handler = ops_int;
	ops_act.sa_flags = SA_RESTART;
	sigemptyset(&ops_act.sa_mask);
	sigaction(SIGINT, &ops_act, &old_act);	// set new, save old
}

void restore_int(void)
{
	sigaction(SIGINT,&old_act,NULL);	// restore old

}

void ops_int(int i)	// interrupt handler
{
	aborted = 1;
}
