; $Id: program_gp32.s,v 1.2 2001/08/17 11:54:42 rwuest Exp $
;
; SPGMR08 68HC08GP32 Loader
; Program flash memory
; Copyright (C) 2001  Sirius Engineering Company
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License version2.
; See the file COPYING that is distributed with this program.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; Block program flash memory
; per algorithm:
; Motorola doc. no. MC68HC908GP32/H  REV. 4, pg 168
; Cpu speed = 2.476 MHZ

	cpu 68HC08
	include gp32


BUF_BASE equ $80
ROW_SIZE equ $40
VER_OFFSET: equ 128			; add to count (in A ) to verify, not program
	
	org BUF_BASE
rdata:  rmb ROW_SIZE		; data to write buffer
dest:   rmb 2
place:  rmb 1
count:  rmb 1
vdest:  rmb 2
vcount: rmb 1


	org $100
	
; program
; A determines function
; A = 0
;	load a row of data to buffer, len (<=row size) in X
;	returns checksum in A:X
;
; A != 0
;	program A bytes from buf to address in H:X
; 	returns A=0 if data verified OK.
;	Some other value if not

program:
	tsta
	beq load_row
	sthx dest
	sthx vdest
	
	cmp #VER_OFFSET
	bls	program_row
	sub	#VER_OFFSET
	sta vcount
	bra verify

; Flash program per algorithm:
; Motorola doc. no. MC68HC908GP32/H  REV. 4, pg 169
; Cpu speed = 2.476 MHZ

program_row:
	sta count
	sta vcount
	lda #rdata
	sta place
	
	lda #$01
	sta FLCR	; step 1, set PGM bits
	lda FLBPR	; step 2, Read the FLBPR
	sta ,X		; step 3, write any data to FLASH block
	lda #8
	dbnza	*	; step 4, 3*8 cycle delay, delay 10 us
	lda #$9	
	sta	FLCR	; step 5, set HVEN
	lda #4
	dbnza	*	; step 6, 3*4 cycle delay, delay 5 us
prog100:
; perform a block copy from row_buf to *dest
	ldx place
	clrh
	lda ,X
	ldhx dest
	sta ,X		; step 7, data to FLASH address
	aix #1
	sthx dest
	inc place	
	lda #24
	dbnza	*	; step 8, 3*24 cycle delay, delay 30 us
	dbnz count,prog100	; step 9, loop til done
	
	lda #$08
	sta FLCR	; step 10, clear PGM
	lda #4
	dbnza	*	; step 11, 3*4 cycle delay, delay 5 us
	clra
	sta FLCR	; step 12, clear HVEN
	lda #1
	dbnza	*	; step 13, 3*1 cycle delay, step 11, delay 1 us

; now verify
; perform a block compare from row_buf to *vdest

verify:
	lda #rdata
	sta place
ver100:	
	ldx place
	clrh
	lda ,X
	ldhx vdest
	cmp ,X
	bne ver200
	aix #1
	sthx vdest
	inc place	
	dbnz vcount,ver100	; loop til all compared
	clra
	bra ver900
ver200:
	lda #$ff
ver900:
	swi
	
load_row:
	tstx
	beq bye
	cpx #ROW_SIZE
	bhi bye
	psha	; init checksum (=0) low byte
	psha	; checksum high byte
	pshx	; save count
	clrh
	ldx #rdata	; data buffer address

load100:
	bsr inch
	sta ,X
	add 3,SP
	sta 3,SP
	bcc load200
	inc 2,SP
load200:
	incx
	dbnz 1,SP,load100
	
	pula	; garbage
	pula	; checksum high byte returned in A
	pulx	; checksum low byte returned in X
bye:	
	swi

; bit banged serial receiver
inch:
	bclr 0,DDRA
in100:
	brset 0,PTA,in100	; wait for start bit
	bsr inbit			; read it
	bcs inch			; noise
	lda #$80			; bit counter
in200:
	bsr inbit			; read bit
	rora				; 1, roll it in
	nop
	bcc in200			; 3, done?
	brclr 0,PTA,*		; 4, stop bit is a one
;	bsr inbit			; 4, get stop bit
;	bcc    framerror	; stop bit is a one
;framerror:
	rts
		
; Wait for a bit at PTA:0
; return it in carry
; 1 bit@9600=104.2 us
; 104.2us @ 2.476MHZ = 257.9 cycles
inT2 equ $17
inbit:
	pshx				; 2
    psha				; 2
    lda #$10			; 2
    ldx #inT2			; 2
inb100:
	brset 0,PTA,inb200	; 5 PTA:0 -> Carry
inb200:
	sbc #0				; 2, A = A - 0 - C
    dbnzx inb100		; 3
    rola				; 2
    pula				; 2
    pulx				; 2
    rts					; 2

	
	end program

	
