// $Id: s1utils.c,v 1.6 2003/12/16 21:23:14 rwuest Exp $
/*
    SPGMR08 68HC08GP32 Loader
    Copyright 1988-2001, Sirius Engineering Company
    Copyright 1995-1997 Kemet Electronics Corporation
    Permission to reuse is granted, but the copyright message
    must stay intact.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/



#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <malloc.h>
#include <errno.h>
#include <string.h>
#include "s1utils.h"
#include "spgmr.h"

#ifdef __TURBOC__
	#include <dir.h>
#endif

int fprints0(FILE *f,char *message)
{
	unsigned char *hmessage = (unsigned char*)message;
	unsigned char csum = 0xff;
	int i;
	if ( hmessage == NULL ) {
		hmessage = (unsigned char *)"SPGMR";
	}
	for ( i=0; hmessage[i]; i++ )
		csum -= hmessage[i];
	i+=3;
	csum -= i;
	fprintf(f,"S0%02x0000",i&0xff);	// put prefix and count
	for ( i=0; hmessage[i]; i++ )
		fprintf(f,"%02x",hmessage[i]&0xff);
	fprintf(f,"%02x\n",csum&0xff);

	return(0);
}

int fprints9(FILE *f,unsigned int entry_address) // Endian trouble ahead
{
	unsigned char csum = 0xff;
	union {
		unsigned int a;
		char c[2];
	}x;
	x.a = entry_address;
	csum -= x.c[0];
	csum -= x.c[1];
	csum -= 3;
	fprintf(f,"S903%02x%02x%02x\n",x.c[0]&0xff,x.c[1]&0xff,csum&0xff);
	return(0);
}

unsigned long fprints1(FILE *f,
                       char *data,
                       unsigned long start_add,
                       unsigned long len)
{
	unsigned long cur_add,remain,this_count;
	unsigned int csum = 0xff;
	unsigned long i;
	cur_add = 0;
	remain = len;

	while (remain) {
		csum = 0xff;
		this_count = (remain>MAXS1WBYTES) ? MAXS1WBYTES : remain;
		remain -= this_count;
		fprintf(f,"S1%02lX%04lX",(this_count+3)&0xFF,start_add);
		csum -= (this_count+3)&0xFF;
		csum -= start_add & 0xff;
        csum -= (start_add >> 8) & 0xff;
		start_add += this_count;
		for ( i = cur_add; i <(cur_add+this_count); i++ ) {
			fprintf(f,"%02X",data[(unsigned)i]&0xFF);
			csum -= (data[i]&0xff);
		}
		cur_add += this_count;
		fprintf(f,"%02X\n",csum&0xff);
	}
	return(start_add);
}

char *gets0header(char *s0buffer)
{
	int i=0;
	char *temp = &s0buffer[2];
	char *s0header = s0buffer;
	int len = gethex(&temp,2)-1;

	if ( !len ) {
		*s0buffer = '\0';
		return(0);
	}
	while ( len-- && !(i=gethex(&temp,2)) )
		;	// skip leading 00's
	if ( !len ) {
		*s0buffer = '\0';
		return(0);
	}
	*s0header = i&0x7f;
	i = 1;
	while ( len-- )
		s0header[i++] = gethex(&temp,2)&0x7f;
	s0header[i] = '\0';
	return s0buffer;
}

unsigned int gethex(char *(*sp),int n)
{
	char c;
	char *p = *sp;
	unsigned int i = 0;

	while (n--) {
		if ((c = *p & 0x7f) < ' ')
			break;
		++p;
		if ((c >= '0') && (c <= '9'))
			i = (i << 4) | (c - '0');
		else
			if (((c &= 0x5f) >= 'A') && (c <= 'F'))
			i = (i << 4) | (c - 'A' + 10);
	}
	*sp = p;
	return (i);
}

/*
	need to pass data in a struct* from s1decodeline
	How about:
	int	type 0,1,9,-1
	int length
	int base
	unsigned char *data;

*/


/* sample line:
S113F0007D0109102600B0B6EC148401102700A776 */

int s1decodeLine(char *line, unsigned char *databuffer,
					  unsigned long *line_address,
					  int *s1_len,
					  enum lcsmode checksum_mode)
{
	unsigned int tempsum,linesum;
	unsigned long s1addr;
	char *p;
	unsigned char *data;
	int i;
	int return_type = S1_NO_DATA;
	static char s1line[MAXS1LINE];

	p = line;

	if ( !databuffer )
		return( S1_BAD_BUFFER);

	if ( *p++ != 'S' )
		return S1_BAD_FORMAT;	// bomb if definitely not a good format

	switch ( *p++ ) {
	case '1':
		return_type = 1;
		break;
	case '5':
		return_type = 5;
		break;
	case '9':
		return_type = 9;
		break;
	case '0':
//		strcpy(s0header,s1line);
//		gets0header(s0header); *//* converts S1 data to text in place */
		return_type = 0;
		break;
	default:
		return(S1_BAD_FORMAT);	// bomb if definitely not a good format
	}

	*s1_len = gethex(&p,2) - 3;		/* get number of bytes in line */

	data = databuffer;

	tempsum = ((*s1_len)+3)&0xff;		/* reset checksum */

	*line_address = s1addr = gethex(&p,4);
	tempsum += (unsigned)((s1addr & 0xff) + (s1addr >> 8));

	for (i = 0; i < (*s1_len); ++i) {	  /* process line */
		data[i] = gethex(&p,2);			/* build data array               */
		tempsum += data[i];				/* and update the checksum         */
	}

	tempsum &=0xff;				/* mask checksum */
	tempsum = 0xff-tempsum;		/* this should be the checksum */
	linesum = gethex(&p,2);		/* get checksum on this line */

	/* verify checksums */
	if ((checksum_mode==LCS_NORMAL) && (tempsum != linesum) ) {
		eprint( "checksum should be %02X: %s\n",
				  tempsum,line);
		return(S1_BAD_CHECKSUM);
	}
	return(return_type);
}
//struct s1_ls {
//	char *s1line;
//	char *data;
//	int type;
//	unsigned long address;
//	int s1len;
//	unsigned int checksum;
//	int flags;
//};

int s1decode(struct s1_ls *s1l)
{
	unsigned int tempsum,linesum;
	unsigned long s1addr;
	char *p;
	unsigned char *data;
	int i;
	int return_type = S1_NO_DATA;

	p = s1l->s1line;

	if ( !(s1l->data) )
		return( S1_BAD_BUFFER);

	if ( *p++ != 'S' )
		return S1_BAD_FORMAT;	// bomb if definitely not a good format

	s1l->type = S1_BAD_FORMAT;	// just in case

	switch ( *p++ ) {
	case '1':
		return_type = 1;
		break;
	case '5':
		return_type = 5;
		break;
	case '9':
		return_type = 9;
		break;
	case '0':
//		strcpy(s0header,s1line);
//		gets0header(s0header); *//* converts S1 data to text in place */
		return_type = 0;
		break;
	default:
		return(S1_BAD_FORMAT);	// bomb if definitely not a good format
	}

	s1l->type = return_type;

	s1l->s1len = gethex(&p,2) - 3;		/* get number of bytes in line */

	data = s1l->data;

	tempsum = ((s1l->s1len)+3)&0xff;		/* reset checksum */

	s1l->address = s1addr = gethex(&p,4);
	tempsum += (unsigned)((s1addr & 0xff) + (s1addr >> 8));

	for (i = 0; i < (s1l->s1len); ++i) {	  /* process line */
		data[i] = gethex(&p,2);			/* build data array */
		tempsum += data[i];				/* and update the checksum */
	}

	tempsum &=0xff;				/* mask checksum */
	tempsum = 0xff-tempsum;		/* this is the checksum */

	s1l->checksum = tempsum;	/* save it for returning  */
	linesum = gethex(&p,2);		/* get checksum on this line */

	/* verify checksums */
	if (( s1l->flags == LCS_NORMAL) && (tempsum != linesum) ) {
		eprint( "checksum should be %02X: %s\n",
				  tempsum,s1l->s1line);
		return(S1_BAD_CHECKSUM);
	}
	return(return_type);
}

/* fill_s1_buffer reads from f and tries to fill it with sequential data
  from multiple lines from the source file
  a maximum of MAXS1DATA bytes will be processed */

char hold_line[MAXS1LINE];
unsigned char hold_data[32];
char *holding=NULL;

int s1ffillBuffer(
						FILE *f,	 /* where to get the data */
						char *databuffer,	 /* where to put data */
						unsigned long *line_address,	  /* return start address here */
						int *s1_len,	/* return actual databuffer byte count here */
						enum lcsmode checksum_mode)
{
	unsigned long work_address; /* work vars */
	int work_len,hold_len;
	unsigned long next_address=0l;
	int pass = 1;	 /* flag to init next_address on first pass */
//  int decode_result;
	int i;

	int done = 0;

	*s1_len = 0;	 /* start by delaring databuffer empty */
	hold_len = 0;
	/* fill buffer on entry, but only if we didn't leave a line last time*/
	if ( !holding ) {
		holding = fgets(hold_line,MAXS1LINE,f);
//		decode_result = s1decodeline(hold_line,hold_data,&work_address,&work_len)
	}
	if ( holding ) {	 /* check that the file had something to read */
		while ( !done ) {
		/* hold line has data */
			switch ( s1decodeLine(hold_line,
										 hold_data,
										 &work_address,
										 &work_len,
										 checksum_mode) ) {
			case 0:	 /* just ignore */
				*s1_len = hold_len;
				break;
			case 1:		/* this is what we want, data! */
				if ( pass == 1 ) {
					pass = 0;	/* let's not do this again */
					next_address = work_address;
					*line_address = work_address;
				}
				/* does it belong and will it fit? */
				if ( (work_address==next_address) && ((*s1_len+work_len)<MAXS1DATA)  ) {
					for ( i=0; work_len; i++ ) {	/* append hold_data to databuffer */
						databuffer[(*s1_len)++] = hold_data[i];
						work_len--;		 /* decrement byte count */
						next_address++;  /* keep track of next */
					}
				}
				else { /* gotta get out of loop and return this data */
					done = 1;
				}
				hold_len = *s1_len;
				break;
			case 9:		/* just ignore */
				default:		/* just ignore */
				*s1_len = hold_len;
				break;
			} /* end of switch s1 line type */
			if ( !done )
				holding = fgets(hold_line,MAXS1LINE,f);
			if ( !holding )
				done = 1;
		} /* end of while not done loop */
	}
	return *s1_len; /* will return 0 if we got no data */
}

/****************************************************************************/
/*int s1fsearchBLk(FILE *f, unsigned char *dest, \                          */
/*                 unsigned long first, unsigned long last)                 */
/*                                                                          */
/* Searches an s1 file, f, for data in the range first:last (inclusive)     */
/* Returns:                                                                 */
/*         count on success.                                                */
/*          may be less than the full area                                  */
/*        -1 on a system or file error                                      */
/*        -2 duplicate results found                                        */
/****************************************************************************/
int s1fsearchBLk(FILE *f, char *dest, \
				 unsigned long first, unsigned long last)
{
	long pos_now;
	char lbuffer[MAXS1LINE];
	unsigned char dbuffer[MAXS1BYTES];
	unsigned long line_address;
	int s1len;
	int s1type;
	int ii;
	unsigned int add;
	int dlen;
	int count;
	unsigned char *dwork;

	dlen = last-first+1;
	count = 0;

	if ( (dwork=(unsigned char *)malloc(dlen*2)) == NULL ) {
		return -1;
	}
	for (ii=dlen; ii<dlen*2; ii++) {
		dwork[ii] = '\0';
	}
//int s1decodeline (char *line, unsigned char *databuffer,
//		  unsigned long *line_address, int *s1len,
//		  enum lcsmode checksum_mode);

	pos_now = ftell(f);
//	rewind( f );
	// search through the file looking for locations first-last

	while ( fgets(lbuffer,MAXS1LINE,f) != NULL ) {
		s1type=s1decodeLine (lbuffer,dbuffer,&line_address,&s1len,LCS_NORMAL);
		if ( s1type == 1 ) { 	// a data line
			// is there interesting data here?
			//  if the end of the record is >= the first address
			// and
			//   the start of the record is <= the last address
			unsigned long end = line_address + s1len;
			if ( (end >= first) && (line_address <= last) ) {
				// got some, now to map the data to 0 based dwork
				for ( ii=0, add=line_address; ii<s1len; ii++, add++ ) {
					if ( (add >= first) && (add <= last) ) {
						int jj = add - first;
						dest[jj] = dbuffer[ii];
						dwork[jj+dlen] = 1;			// flag as found
						count ++;
					}
				}
			}
		}
	}
	// copy data
//	memcpy( dest, dwork, dlen);
	// reset file position
	fseek( f, pos_now, SEEK_SET);

	free( dwork );

	return count;
}


// $Log: s1utils.c,v $
// Revision 1.6  2003/12/16 21:23:14  rwuest
// changed unsigned chars to chars
//
// Revision 1.5  2001/08/17 11:54:42  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.4  2001/08/04 17:15:26  rwuest
// fixed checksums in fprintfs0() and fprintfs9
//
// Revision 1.3  2001/07/26 21:04:11  rwuest
// renamed everything
// Added s1fsearchBLk()
//
// Revision 1.2  2001/07/22 00:37:08  rwuest
// Added Log messages at end
//




