// $Id: seccodes.c,v 1.7 2003/12/16 21:19:31 rwuest Exp $
/*
    SPGMR08 68HC08GP32 Loader
	handles for security codes
    Copyright (C) 2000  Robert Wuest (rwuest@wuest.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
    See the file COPYING that is distributed with this program.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "spgmr.h"
#include "seccodes.h"
#include "s1utils.h"


unsigned char xcodes[SECCSIZE];
unsigned char sec_codes[SECCMAX][SECCSIZE] = {
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff},
	{0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff}
};

/****************************************************************************/
/*                 int scodes_init(void)                                    */
/*                                                                          */
/* Opens and mmaps the scodes rc file                                       */
/* It is unlikely this is portable.                                         */
/* This uses optional command line parameter "-S filename" to locate the    */
/* desired security codes file.                                             */
/* Pertinent global variables:                                              */
/*   extern int Sflag = 0;                                                  */
/*   extern FILE *Sfile = 0;                                                */
/*   extern char *Sfilename = 0;                                            */
/****************************************************************************/
int scodes_init(void)
{
	char *sdir;
	int ii;

	if ( !Sflag ) {				// set when -S is passed on command line
		sdir = getenv("HOME");		// get users home dir for default file
		if ( !sdir ) {
			eprint( "sdir not found\n");
		}
		ii = strlen( SECCFILE ) + strlen(sdir) + 1;

	}
	return 0;
}

/****************************************************************************/
/*                 int scodes_close(void)                                   */
/*                                                                          */
/****************************************************************************/
int scodes_close(void)
{
	return 0;
}
/****************************************************************************/
/*                 int scodes_find_s1(FILE *f, int which)                   */
/*                                                                          */
/* search through the file looking for locations $FFF6$FFFD                */
/* returns 0 on success, -1 if which is out of bounds                       */
/****************************************************************************/
int scodes_find_s1(FILE *f, int which, char *buf)
{
	int ii;
	
	memset( buf, 0xff, SECCSIZE );
	
	if ( which>=0 && which<SECCMAX ) {
		ii = s1fsearchBLk( f, buf, 0xfff6, 0xfffd);
		return 0;
	} else {
		return -1;
	}
}


/****************************************************************************/
/*                 int scodes_set( char *codes, int which )                 */
/*                                                                          */
/* Copies SECCSIZE bytes from codes to sec_codes array[which]               */
/* returns 0 on success, -1 if which is out of bounds                       */
/****************************************************************************/
int scodes_set( char *codes, int which )
{
	int ii;
	int rv =0;

	if ( which>=0 && which<SECCMAX ) {
		for (ii=0; ii<SECCSIZE; ii++) {
			sec_codes[which][ii] = codes[ii];
		}
		return 0;
	} else {
		return -1;
	}
}


/****************************************************************************/
/*                 int scodes_get( char *codes, int which )                 */
/*                                                                          */
/* Copies SECCSIZE bytes from sec_codes array[which] to codes               */
/* returns 0 on success, -1 if which is out of bounds                       */
/****************************************************************************/
int scodes_get( char *codes, int which )
{
	int ii;
	int rv =0;

	if ( which>=0 && which<SECCMAX ) {
		for (ii=0; ii<SECCSIZE; ii++) {
			codes[ii] = sec_codes[which][ii];
		}
		return 0;
	} else {
		return -1;
	}
}


/****************************************************************************/
/*                 void scodes_push( char *codes )                          */
/*                                                                          */
/* Pushes the codes in buf onto the sec_codes array                         */
/* will not push a duplicate entry                                          */
/* Always succeeds, no return value                                         */
/* this is really inefficient, maybe should use array of pointers           */
/****************************************************************************/
void scodes_push( char *codes )
{
	int ii;
	char tcodes[SECCSIZE];
	// first compare buf to sec_codes[0]
	// and return if they match
	for ( ii=0; (ii<SECCSIZE) && (codes[ii]==sec_codes[0][ii]); ii++ )
		;
	if ( ii == SECCSIZE ) { // true when they match
		return;
	}

	// move em all down one
	// start at bottom and work up

	for ( ii=SECCMAX-2; ii>=0; ii-- ) {
		scodes_get( tcodes, ii );
		scodes_set( tcodes, ii+1 );
	}
	scodes_set( codes, 0 );
}


/****************************************************************************/
/*                 int scodes_print( int which )                            */
/*                                                                          */
/* Prints scodes with xprint.  Which is the codes to print, -1 to print all */
/* Returns -1 if which is out out bounds.                                   */
/****************************************************************************/
int scodes_print( int which )
{
	int ii;
	int rv =0;

	if ( which>=0 && which<SECCMAX ) {
		for( ii=0; ii<SECCSIZE; ii++ )
			xprint( "%02X ", sec_codes[which][ii] );
		xprint("\n");

	} else if ( which == XCODES ) {
		for( ii=0; ii<SECCSIZE; ii++ )
			xprint( "%02X ", xcodes[ii] );
		xprint("\n");
	}
	else
		return -1;

	return 0;

}



// $Log: seccodes.c,v $
// Revision 1.7  2003/12/16 21:19:31  rwuest
// changed unsigned chars to chars
//
// Revision 1.6  2001/08/17 11:54:43  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.5  2001/08/04 16:37:52  rwuest
// major rewrite
//
// Revision 1.4  2001/07/29 03:37:07  rwuest
// added set_scodes to store data from verify/program buffer to sec_codes
// array
//
// Revision 1.3  2001/07/26 20:57:59  rwuest
// Use s1fsearchBLk() to find the codes.
//
// Revision 1.2  2001/07/22 00:29:06  rwuest
// Debug messages now all print through dprint()
// Standard out now all goes througn xprint()
// Error messages all go through eprint()
// added log messages at end
//
