// $Id: utils.c,v 1.8 2003/12/16 21:19:31 rwuest Exp $
/*
	SPGMR08 68HC08GP32 Loader
	Copyright (C) 2001  Sirius Engineering Company

	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2.
	See the file COPYING that is distributed with this program.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
*/

//#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/stat.h>

#include "spgmr.h"

// Miscellaneous stuff

/****************************************************************************/
/*                 int xprint(const char *fmt, ...)                         */
/*                                                                          */
/* This is a replacement printf, sending the output to outfile and flushing */
/* the output afterwards                                                    */
/* the return value is the number of characters printed                     */
/****************************************************************************/
int xprint(const char *fmt, ...)
{
	int rv;
	va_list ap;

	va_start( ap, fmt );

#if 0
	rv = vdprintf( STDOUT_FILENO, fmt, ap );
	fsync( STDOUT_FILENO );
#else
	rv = vfprintf( outfile, fmt, ap );
	fflush( outfile );
#endif

	va_end( ap );
	return rv;
}


/****************************************************************************/
/*                 int eprint(const char *fmt, ...)                         */
/*                                                                          */
/* Print error message                                                      */
/* Use this atomically (ie, don;t build output with multiple calls)         */
/* This is a replacement printf, sending the output to errfile              */
/* the return value is the number of characters printed                     */
/****************************************************************************/
int eprint(const char *fmt, ...)
{
	int rv;
	va_list ap;
	va_start( ap, fmt );

	rv = fprintf( errfile, "error: " );
	rv += vfprintf( errfile, fmt, ap );
	fflush( errfile );

	va_end( ap );
	return rv;
}

/****************************************************************************/
/*                 int wprint(const char *fmt, ...)                         */
/*                                                                          */
/* Print warning message                                                    */
/* Use this atomically (ie, don;t build output with multiple calls)         */
/* This is a replacement printf, sending the output to errfile              */
/* the return value is the number of characters printed                     */
/****************************************************************************/
int wprint(const char *fmt, ...)
{
	int rv;
	va_list ap;
	va_start( ap, fmt );

	rv = fprintf( errfile, "warning: " );
	rv += vfprintf( errfile, fmt, ap );
	fflush( errfile );
	
	va_end( ap );
	return rv;
}


/****************************************************************************/
/*                 int dprint(const char *fmt, ...)                         */
/*                                                                          */
/* This is a replacement printf, sending the output to dbgfile              */
/* the return value is the number of characters printed                     */
/****************************************************************************/
int dprint(const char *fmt, ...)
{
	int rv=0;
	va_list ap;
	if( debug ) {
		va_start( ap, fmt );
		rv = vfprintf( dbgfile, fmt, ap );
	
		va_end( ap );
	}
	return rv;
}


/****************************************************************************/
/*                 void format_dump(char *buf, int base, int len)           */
/*                                                                          */
/* Print puffer in pretty formatted hex with addresses                      */
/* and ASCII (if printable)                                                 */
/* Always succeeds, nothing returned.                                       */
/****************************************************************************/
#define CPL 16

void format_dump(char *buf, int base, int len)
{
	int i,j,k,l;
	unsigned long place = base;
	unsigned long fillplace = base & 0xfff0;

	i=0;
	l = 0;

	if( !buf | !len )	// no valid data - out of here
		return;

	while (i < len ) {
		xprint( "%04lX ", place );
		for ( j=0; j<CPL && (i+j) < len; j++ ) {
			xprint( "%02x", buf[i+j]&0xff );
		}
		place += j;
		if ( l && j != CPL ) {
			while ( j<CPL ) {
				xprint("  ");
				j++;
			}
			xprint(" | ");
		}
		xprint(" | ");

		for ( j=0; j<CPL && (i+j) < len ; j++ ) {
			xprint( "%c", isprint(buf[i+j])?buf[i+j]:'.' );
		}
		xprint("\n");
		l++;			// l counts lines
		i+=CPL;
	}

}


/****************************************************************************/
/*                 void statinfo(int fd )                                   */
/*                                                                          */
/* This is a debugging routine to help get the server mode working          */
/****************************************************************************/
void statinfo(int fd )
{
	struct stat stat_buf;

	fstat( fd, &stat_buf );
	xprint( "FD: %-15d ", fd );
	xprint( "inode: %-12d ", stat_buf.st_ino );
	xprint( "mode: 0%o\n", stat_buf.st_mode );
	xprint( "dev: %-14d ", stat_buf.st_dev );
	xprint( "rdev: %-13d ", stat_buf.st_rdev );
	xprint( "nlink: %d\n", stat_buf.st_nlink );
	xprint( "uid: %-14d ", stat_buf.st_uid );
	xprint( "gid: %d\n", stat_buf.st_gid );
	xprint( "size: %-13d ", stat_buf.st_size );
	xprint( "blksize: %-10d ", stat_buf.st_blksize );
	xprint( "blocks: %d\n", stat_buf.st_blocks );
	xprint( "atime: %-12d ", stat_buf.st_atime );
	xprint( "mtime: %-12d ", stat_buf.st_mtime );
	xprint( "ctime: %-12d\n", stat_buf.st_ctime );


}


/****************************************************************************/
/*                 char *[lr ]trim(char *)                                  */
/*                                                                          */
/* Remove whitespace at the ends of text                                    */
/* ltrim removes left (leading) whitespace                                  */
/* rtim removes right (trailing) whitespace                                 */
/* trim does both ends                                                      */
/* Returns the passed pointer                                               */
/****************************************************************************/
char *ltrim(char *s)
{
	int i,j;
	char *t,*u;
	if( s == NULL || *s == '\0' )
		return s;

	u = t = s;
	while( isspace(*t) )
		t++;
	while( *t )
		*s++=*t++;
	*s = '\0';
	return u;
}

char *rtrim(char *s)
{
	int i = strlen(s);
	char *t = &s[i-1];

	if( s == NULL || *s == '\0' )
		return s;

	while( isspace(*t) )
		*t-- = '\0';

	return s;
}




char *trim(char *s)
{

	if( s == NULL || *s == '\0' )
		return s;

	return ltrim(rtrim(s));
}



// $Log: utils.c,v $
// Revision 1.8  2003/12/16 21:19:31  rwuest
// changed unsigned chars to chars
//
// Revision 1.7  2001/08/17 11:54:43  rwuest
// Modify license to eliminate the "any later version clause"
// If a later version comes out, and I want to use it,
// I'll change the license then.
//
// Revision 1.6  2001/08/04 17:07:57  rwuest
// cleanup xprint() - kindof commited to using vfprintf()
// removed any error echoing
// added wprint()
// eprint() prepends "error: " and wprint() prepends "warning: "
//
// Revision 1.5  2001/07/29 03:35:56  rwuest
// back to using vdprintf( STDOUT_FILENO... for xprint trying to find pipe
// errors talking to tkspgmr. seems ok now.
//
// Revision 1.4  2001/07/27 16:48:16  rwuest
// dprint is now it's own function
// fileinfo through xprint
// trying vdprintf( STDOUT_FILENO,... in xprint for progress in tkspgmr
//
// Revision 1.3  2001/07/26 20:51:04  rwuest
// Added trim functions, copied from rwlib
//
// Revision 1.2  2001/07/22 02:17:54  rwuest
// Don't print I/O messages when not interactive or not debug
//
// Revision 1.1  2001/07/22 00:37:08  rwuest
// New file with print functions
// Debug messages now all print through dprint()
// Standard out now all goes througn xprint()
// Error messages all go through eprint()
//

